#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp                      import conf
from pyforms_gui.controls.control_base import ControlBase

import pyforms_gui.utils.tools as tools


from AnyQt           import uic, _api
from AnyQt.QtWidgets import QFileDialog


class ControlFile(ControlBase):

    def __init__(self, *args, **kwargs):
        self.__exec_changed_event = True
        super(ControlFile, self).__init__(*args, **kwargs)
        self.use_save_dialog = kwargs.get('use_save_dialog', False)

        # used to avoid triggering the finnishEditing when the input loses the focus
        self._value_before_finishEditing = None

    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "fileInput.ui")
        self._form = uic.loadUi(control_path)
        self._form.label.setText(self._label)
        self._form.pushButton.clicked.connect(self.click)
        self.form.lineEdit.editingFinished.connect(self.finishEditing)
        self._form.pushButton.setIcon(conf.PYFORMS_ICON_FILE_OPEN)
        super().init_form()

    def finishEditing(self):
        """Function called when the lineEdit widget is edited"""
        if self.__exec_changed_event and self._value_before_finishEditing!=self.value:
            self._value_before_finishEditing = self.value
            self.changed_event()

    def click(self):
        if self.use_save_dialog:
            value, _ = QFileDialog.getSaveFileName(self.parent, self._label, self.value)
        else:
            if conf.PYFORMS_DIALOGS_OPTIONS:
                value = QFileDialog.getOpenFileName(self.parent, self._label, self.value,
                                                    options=conf.PYFORMS_DIALOGS_OPTIONS)
            else:
                value = QFileDialog.getOpenFileName(self.parent, self._label, self.value)


        if _api.USED_API == _api.QT_API_PYQT5:
            value = value[0]
        elif _api.USED_API == _api.QT_API_PYQT4:
            value = str(value)

        if value and len(value)>0:
            self.value = value
            self._value_before_finishEditing = value


    @property
    def value(self):
        self._value = str(self._form.lineEdit.text())
        return self._value

    @value.setter
    def value(self, value):
        self.__exec_changed_event = False
        self._form.lineEdit.setText(value)
        self.__exec_changed_event = True
        ControlBase.value.fset(self, value)

    @property
    def label(self): return self.form.label.text()

    @label.setter
    def label(self, value):
        self.form.label.setText(value)
        ControlBase.label.fset(self, value)


# !/usr/bin/python
# -*- coding: utf-8 -*-

from confapp      import conf

from AnyQt           import QtCore, uic
from AnyQt.QtWidgets import QMenu, QAction
from AnyQt.QtGui     import QIcon, QKeySequence

class ControlBase(object):
    """
    All the Controls inherit from this Control, therefore you can find its functions and properties in all the other controls listed below.

    """

    def __init__(self, *args, **kwargs):
        """
        :param str label: Control label. Default = ''.
        :param str helptext: Text shown when the mouse is over the control. Default = None.
        :param str default: Initial value of the control. Default = None.

        :param bool visible: Flag to set the control visible or hidden. Default = True.
        :param bool enabled: Flag to set the control enabled or Disabled. Default = True.
        :param bool readonly: Flag to set the control readonly. Default = False.
        :param function changed_event: Function to call whenever the control value is updated. Default = None.  

        """

        self._form       = None  # Qt widget
        self._parent     = None  # Parent window
        self._popup_menu = None

        self._help          = kwargs.get('helptext', None)
        self._value         = kwargs.get('default',  None)
        self._label         = kwargs.get('label', args[0] if len(args)>0 else '')
        self._style         = kwargs.get('style', None)

        self.init_form()

        self.changed_event  = kwargs.get('changed_event', self.changed_event)
        self.enabled        = kwargs.get('enabled', True)
        self.readonly       = kwargs.get('readonly', False)
        if not kwargs.get('visible', True):  self.hide()


    def __repr__(self): return str(self._value)

    ##########################################################################
    ############ Funcions ####################################################
    ##########################################################################

    def init_form(self):
        """
        Load the control UI and initiate all the events.
        """     
        if self.help: self.form.setToolTip(self.help)

        if self._style: self.form.setStyleSheet(self._style)
   


    def load_form(self, data, path=None):
        """
        Loads the value of the control.

        :param dict data: It is a dictionary with the required information to load the control. 
        :param str path: Optional parameter that can be used to save the data.  
        """
        if 'value' in data:
            self.value = data['value']

    def save_form(self, data, path=None):
        """
        Save a value of the control to a dictionary.  

        :param dict data: Dictionary where the control value should be saved.  
        :param str path: Optional parameter that can be used to load the data.  
        """
        data['value'] = self.value
        return data

    def show(self):
        """
        Show the control
        """
        if self.form is None:
            return
        elif self.form==self:
            super(ControlBase,self).show()
        else:
            self.form.show()

    def hide(self):
        """
        Hide the control
        """
        if self.form is None:
            return
        elif self.form==self:
            super(ControlBase,self).hide()
        else:
            self.form.hide()


    def __create_popup_menu(self):
        if not self._popup_menu:
            self.form.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
            self.form.customContextMenuRequested.connect(self._open_popup_menu)
            self._popup_menu = QMenu(self.parent)
            self._popup_menu.aboutToShow.connect( self.about_to_show_contextmenu_event)

    def add_popup_submenu(self, label, submenu=None):
        """
        It returns a new sub popup menu. If submenu is open the menu is added to the main popup menu.

        :param str label: Label of the option  
        :param QMenu submenu: Parent submenu to which the option should be added. If no value is set, then the option will be added to the main popup menu.  
        """
        self.__create_popup_menu()
        menu = submenu if submenu else self._popup_menu
        submenu = QMenu(label, menu)
        menu.addMenu(submenu)
        return submenu

    def add_popup_menu_option(self, label, function_action=None, key=None, icon=None, menu=None):
        """
        Add an option to the Control popup menu.  

        :param str label: Label of the option  
        :param function function_action: The function that should be executed when the menu is selected.  
        :param str key: Short key.  
        :param QIcon or str icon: Icon.  
        :param QMenu submenu: Parent submenu to which the option should be added. If no value is set, then the option will be added to the main popup menu.  
        
        .. code:: python

            control.add_popup_menu_option('option 0', function_action=self._do_something)
            submenu1 = control.add_popup_submenu('menu 1')
            submenu2 = control.add_popup_submenu('menu 2', submenu=submenu1)
            control.add_popup_menu_option('option 1', function_action=self._do_something, key='Control+Q', submenu=submenu2)
        """
        self.__create_popup_menu()

        menu = menu if menu else self._popup_menu

        if label == "-":
            return menu.addSeparator()
        else:
            action = QAction(label, self.form)
            if icon is not None:
                action.setIconVisibleInMenu(True)
                action.setIcon(icon if isinstance(icon, QIcon) else QIcon(icon) )
            if key != None:
                action.setShortcut(QKeySequence(key))
            if function_action:
                action.triggered.connect(function_action)
                menu.addAction(action)
            return action


    ##########################################################################
    ############ Events ######################################################
    ##########################################################################

    def changed_event(self):
        """
        Function called when ever the Control value is changed. The event function should return True if the data was saved with success.

        """
        return True

    def about_to_show_contextmenu_event(self):
        """
        Function called before the Control popup menu is opened.
        """
        pass

    def _open_popup_menu(self, position):
        if self._popup_menu:
            self._popup_menu.exec_(self.form.mapToGlobal(position))

    ##########################################################################
    ############ Properties ##################################################
    ##########################################################################

    ##########################################################################
    # Set the Control enabled or disabled

    @property
    def enabled(self):
        """
        Returns or set if the control is enable or disable.
        """
        return self.form.isEnabled()

    @enabled.setter
    def enabled(self, value):
        self.form.setEnabled(value)

    ##########################################################################
    # Return or update the value of the Control

    @property
    def value(self):
        """
        This property returns or set what the control should manage or store.
        """
        return self._value

    @value.setter
    def value(self, value):
        oldvalue = self._value
        self._value = value
        if oldvalue != value:
            self.changed_event()

    
    @property
    def name(self): 
        """
        This property returns or set the name of the control.
        """
        return self.form.objectName()

    @name.setter
    def name(self, value):
        self.form.setObjectName(value)

    ##########################################################################
    # Return or update the label of the Control

    @property
    def label(self):
        """
        Returns or sets the label of the control.
        """
        return self._label

    @label.setter
    def label(self, value):
        self._label = value

    ##########################################################################
    # Parent window

    @property
    def parent(self):
        """
        Returns or set the parent basewidget where the Control is.
        """
        return self._parent

    @parent.setter
    def parent(self, value):
        self._parent = value

    @property
    def visible(self):
        """
        Return the control visibility.
        """
        return self.form.isVisible()

    @property
    def help(self):
        """
        Returns or set the tip box of the control.
        """
        return self._help if self._help else ''

    @property
    def error(self): return None

    @error.setter
    def error(self, value):
        pass

    @property
    def label_visible(self): return None

    @label_visible.setter
    def label_visible(self, value):
        pass

    @property
    def readonly(self):
        """
        Set and return the control readonly state.
        """
        return None

    @readonly.setter
    def readonly(self, value):
        pass

    @property
    def css(self): return None

    @css.setter
    def css(self, value):
        pass


    ##########################################################################
    ############ Properties just for the GUI version #########################
    ##########################################################################

    ##########################################################################
    # Return the QT widget

    @property
    def form(self):
        """
        Returns the QWidget of the control.
        """
        return self._form



# !/usr/bin/python
# -*- coding: utf-8 -*-
 
import logging
 
from confapp import conf
 
logger = logging.getLogger(__name__)
 
from pyforms_gui.controls.control_base import ControlBase
from pyforms_gui.utils import tools
 
try:
    import cv2
except:
    raise Exception('OpenCV is not available. ControlImage will not be working')
 
import OpenGL.GL  as GL
import OpenGL.GLU as GLU
        
from AnyQt           import QtCore, QtOpenGL, uic
from AnyQt.QtWidgets import QWidget
 
from AnyQt import _api
 
if _api.USED_API == _api.QT_API_PYQT5:
    try:
        from AnyQt.QtOpenGL import QGLWidget
    except:
        logger.debug("No OpenGL library available")
 
    import platform
    if platform.system() == 'Darwin':
        from pyforms_gui.controls.control_player.VideoQt5GLWidget import VideoQt5GLWidget as VideoGLWidget
    else:
        from pyforms_gui.controls.control_player.VideoGLWidget import VideoGLWidget
 
elif _api.USED_API == _api.QT_API_PYQT4:
    try:
        from PyQt4.QtOpenGL import QGLWidget
    except:
        logger.debug("No OpenGL library available")
 
    from pyforms_gui.controls.control_player.VideoGLWidget import VideoGLWidget
 
import numpy as np
 
class ControlImage(ControlBase):
    _imageWidget = None
 
    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "image.ui")
        self._form = uic.loadUi(control_path)
        self._imageWidget = VideoGLWidget()
        self._form.imageLayout.addWidget(self._imageWidget)
        super(ControlImage, self).init_form()
 
    def save_form(self, data, path=None):
        if type(self.value) is np.ndarray: data['value'] = self._value


    @property
    def value(self):
        """
        This property returns or set what the control should manage or store.
        """
        return self._value

    @value.setter
    def value(self, value):
        oldvalue = self._value
        self._value = value
        self._imageWidget.paint([value])
        if  (type(oldvalue) is np.ndarray and type(value) is np.ndarray and oldvalue.any()!=value.any()) or \
            (type(oldvalue) is np.ndarray and type(value) is not np.ndarray) or \
            (type(oldvalue) is not np.ndarray and type(value) is np.ndarray):
            self.changed_event()

    ##########################################################################
    ############ EVENTS ######################################################
    ##########################################################################
    
    @property
    def double_click_event(self):
        return self._imageWidget.onDoubleClick

    @double_click_event.setter
    def double_click_event(self, value):
        self._imageWidget.onDoubleClick = value

    @property
    def click_event(self):
        return self._imageWidget.onClick

    @click_event.setter
    def click_event(self, value):
        self._imageWidget.onClick = value

    @property
    def drag_event(self):
        return self._imageWidget.onDrag

    @drag_event.setter
    def drag_event(self, value):
        self._imageWidget.onDrag = value

    @property
    def end_drag_event(self):
        return self._imageWidget.onEndDrag

    @end_drag_event.setter
    def end_drag_event(self, value):
        self._imageWidget.onEndDrag = value

    @property
    def key_release_event(self): return self._imageWidget.on_key_release

    @key_release_event.setter
    def key_release_event(self, value): self._imageWidget.on_key_release = value





class vsplitter(object):
    def __init__(self, *args, **kwargs):
        self.items = args
        self.left_width = kwargs.get('left_width', 1)
        self.right_width = kwargs.get('right_width', 1)
    def __getitem__(self,index): return self.items[index]
    def __setitem__(self,index,value): self.items[index] = value
    def __len__(self):  return len(self.items)
    def __iter__(self): 
        self._index = -1; return self
    def __next__(self): 
        self._index += 1
        if self._index>=len(self.items): raise StopIteration
        return self.items[self._index]

class hsplitter(object):
    def __init__(self, *args, **kwargs):  self.items = args
    def __getitem__(self,index): return self.items[index]
    def __setitem__(self,index,value): self.items[index] = value
    def __len__(self):  return len(self.items)
    def __iter__(self): 
        self._index = -1; return self
    def __next__(self): 
        self._index += 1
        if self._index>=len(self.items): raise StopIteration
        return self.items[self._index]


class no_columns(object):
    def __init__(self, *args):  self.items = args
    def __getitem__(self,index): return self.items[index]
    def __setitem__(self,index,value): self.items[index] = value
    def __len__(self):  return len(self.items)
    def __iter__(self): 
        self._index = -1; return self
    def __next__(self): 
        self._index += 1
        if self._index>=len(self.items): raise StopIteration
        return self.items[self._index]


class segment(object):
    def __init__(self, *args, **kwargs): 
        self.css = kwargs.get('css', '')
        self.items = args
    def __getitem__(self,index): return self.items[index]
    def __setitem__(self,index,value): self.items[index] = value
    def __len__(self):  return len(self.items)
    def __iter__(self): 
        self._index = -1; return self
    def __next__(self): 
        self._index += 1
        if self._index>=len(self.items): raise StopIteration
        return self.items[self._index]



from pyforms_gui.controls.control_player.AbstractGLWidget import AbstractGLWidget

import math
from AnyQt import QtGui
from AnyQt.QtWidgets import QOpenGLWidget
from AnyQt import QtCore


class VideoQt5GLWidget(AbstractGLWidget, QOpenGLWidget):

	def initializeGL(self):
		self.gl = self.context().versionFunctions()
		self.gl.initializeOpenGLFunctions()

		'''
		 Sets up the OpenGL rendering context, defines display lists, etc. 
		 Gets called once before the first time resizeGL() or paintGL() is called.
		'''
		self.gl.glClearDepth(1.0)
		self.gl.glClearColor(0, 0, 0, 1.0)
		self.gl.glEnable(self.gl.GL_DEPTH_TEST)

	def perspective(self, fovy, aspect, zNear, zFar):
		ymax = zNear * math.tan( fovy * math.pi / 360.0 );
		ymin = -ymax;
		xmin = ymin * aspect;
		xmax = ymax * aspect;

		self.gl.glFrustum( xmin, xmax, ymin, ymax, zNear, zFar )

	def resizeGL(self, width, height):
		self.setupViewport(width, height)

	def setupViewport(self, width, height):
		side = min(width, height)
		self.gl.glViewport((width - side) // 2, (height - side) // 2, side,
				side)

		self.gl.glMatrixMode(self.gl.GL_PROJECTION)
		self.gl.glLoadIdentity()
		self.perspective(40.0, float(width) / float(height), 0.01, 10.0)
		self.gl.glMatrixMode(self.gl.GL_MODELVIEW)

	def draw_helptext(self):
		pass

	def draw_message(self):
		pass


#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"





#!/usr/bin/python
# -*- coding: utf-8 -*-

import pyforms_gui.utils.tools as tools

from AnyQt                            import uic
from pyforms_gui.controls.control_base import ControlBase


class ControlSlider(ControlBase):

    def __init__(self, *args, **kwargs):
        self._updateSlider = True
        self._min = kwargs.get('minimum', 0)
        self._max = kwargs.get('maximum', 100)
        if 'default' not in kwargs: kwargs['default'] = 0
        ControlBase.__init__(self, *args, **kwargs)
        
        self._form.value.setText(str(kwargs['default']))
        self._form.horizontalSlider.valueChanged.connect(self.valueChanged)

    

    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "sliderInput.ui")
        self._form = uic.loadUi(control_path)
        self._form.label.setText(self._label)
        self.form.label.setAccessibleName('ControlSlider-label')

        self._form.horizontalSlider.setMinimum(self._min)
        self._form.horizontalSlider.setMaximum(self._max)
        self._form.horizontalSlider.setValue(self._value)
        super(ControlSlider, self).init_form()

    def valueChanged(self, value):
        self._updateSlider = False
        self.value = value
        self._updateSlider = True

    def load_form(self, data, path=None):
        if 'value' in data: self.value = int(data['value'])

    def save_form(self, data, path=None):
        data['value'] = self.value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        ControlBase.value.fset(self, value)
        if self._updateSlider and self._value != None:  self._form.horizontalSlider.setValue(int(value))
        self._form.value.setText(str(value))

    @property
    def min(self):
        return self._form.horizontalSlider.minimum()

    @min.setter
    def min(self, value):
        self._form.horizontalSlider.setMinimum(value)

    @property
    def max(self):
        return self._form.horizontalSlider.maximum()

    @max.setter
    def max(self, value):
        self._form.horizontalSlider.setMaximum(value)



#!/usr/bin/python
# -*- coding: utf-8 -*-


from confapp import conf

from pyforms_gui.controls.control_base import ControlBase

from AnyQt           import QtCore
from AnyQt.QtWidgets import QTreeView, QFileSystemModel

class ControlFilesTree(ControlBase):
    def init_form(self):
        self._form = QTreeView()
        super(ControlFilesTree, self).init_form()

    @property
    def value(self): return self._value

    @value.setter
    def value(self, value):
        ControlBase.value.fset(self, value)
        model = QFileSystemModel(parent=None)
        self._form.setModel(model)
        model.setRootPath(QtCore.QDir.currentPath())

        self._form.setRootIndex(model.setRootPath(value))

        self._form.setIconSize(QtCore.QSize(32, 32))



from idtrackerai.list_of_blobs import ListOfBlobs
from idtrackerai.video import Video
import os

class Chosen_Video(object):
    
    def __init__(self, processes_list = None, **kwargs):
        self.video = kwargs.get('video',  Video() )
        self.list_of_fragments = kwargs.get('list_of_fragments', None)
        self.list_of_global_fragments = kwargs.get('list_of_global_fragments', None)

        if self.video.blobs_path is not None and os.path.exists(self.video.blobs_path):
            self.list_of_blobs = ListOfBlobs.load(self.video, self.video.blobs_path)
        else:
            self.list_of_blobs = None

        self.chosen = 'Default String'
        self.processes_list = processes_list
        self.processes_to_restore = {}
        self.old_video = None
        

    def set_chosen_item(self, chosen_string):
        self.chosen = chosen_string



# !/usr/bin/python
# -*- coding: utf-8 -*-

import bisect
from AnyQt.QtGui import QColor

class Track(object):
	"""
	
	"""

	DEFAULT_COLOR = QColor(100, 100, 255)

	def __init__(self, parent):
		"""
		
		:param parent: 
		"""
		self._title = ''
		self._color = self.DEFAULT_COLOR
		self._parent = parent
		self._events = []

	# Functions needed for the bisect insertion ############################
	def __len__(self):
		return len(self._events)

	def __getitem__(self, index):
		if isinstance(index, slice):
			return [self._events[x] for x in range(*index.indices(len(self)))]
		return self._events[index]

	def insert(self, item, index):
		"""
		
		:param item: 
		:param index: 
		:return: 
		"""
		self._events.insert(item, index)

	########################################################################

	def add_event(self, period):
		"""
		The events are added in a sorted way for rendering optimization
		:param period: 
		:return: 
		"""
		bisect.insort_right(self, period)
		return period

	def draw(self, painter, start, end, track_index):
		"""
		
		:param painter: 
		:param start: 
		:param end: 
		:param track_index: 
		:return: 
		"""
		y = (track_index * self._parent.tracks_height) + 18
		painter.drawLine(start, y, end, y)

	def draw_events(self, painter, start, end, track_index, left_shift=0, scale=1.0):
		"""
		
		:param painter: 
		:param start: 
		:param end: 
		:param track_index: 
		:param left_shift: 
		:param scale: 
		:return: 
		"""
		top = track_index * self._parent.tracks_height + 20

		first_index = bisect.bisect(self, start)

		for period in self._events[first_index:]:
			if period.begin > end:
				continue
			painter.setBrush(period.color)
			period.draw(painter, top=top, left_shift=left_shift, scale=scale)

	def clear(self):
		"""
		
		:return: 
		"""
		del self._events[:]
		self._events = []

	@property
	def events(self):
		return self._events

	@property
	def color(self):
		return self._color

	@color.setter
	def color(self, value):
		self._color = value

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, value):
		self._title = value

	@property
	def properties(self):
		return ['T', self.title, self.color.name()]

	@properties.setter
	def properties(self, value):
		self.title = value[1]
		self.color = QColor(value[2])



import os
from AnyQt.QtGui import QIcon, QPixmap
from AnyQt.QtWidgets import QStyle, qApp


def path(filename):
	"""
	:param filename:
	:return:
	"""
	return os.path.join(os.path.dirname(__file__), filename)


PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY = QIcon()
PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY.addPixmap(qApp.style().standardPixmap(QStyle.SP_MediaPlay), mode=QIcon.Normal,
                                              state=QIcon.Off)
PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY.addPixmap(qApp.style().standardPixmap(QStyle.SP_MediaPause), mode=QIcon.Normal,
                                              state=QIcon.On)

PYFORMS_ICON_VIDEOPLAYER_DETACH = QIcon()
PYFORMS_ICON_VIDEOPLAYER_DETACH.addPixmap(qApp.style().standardPixmap(QStyle.SP_TitleBarMaxButton), mode=QIcon.Normal )

PYFORMS_ICON_CODEEDITOR_SAVE = QIcon(qApp.style().standardPixmap(QStyle.SP_DialogSaveButton))
PYFORMS_ICON_CODEEDITOR_DISCARD = QIcon(qApp.style().standardPixmap(QStyle.SP_DialogDiscardButton))

PYFORMS_PIXMAP_EVENTTIMELINE_ZOOM_IN = QPixmap(path(os.path.join("Controls", "uipics", "zoom_in.png")))
PYFORMS_PIXMAP_EVENTTIMELINE_ZOOM_OUT = QPixmap(path(os.path.join("Controls", "uipics", "zoom_in.png")))

PYFORMS_ICON_EVENTTIMELINE_IMPORT = QIcon(path(os.path.join("Controls", "uipics", "page_white_get.png")))
PYFORMS_ICON_EVENTTIMELINE_EXPORT = QIcon(path(os.path.join("Controls", "uipics", "page_white_put.png")))
PYFORMS_ICON_EVENTTIMELINE_GRAPH = QIcon(path(os.path.join("Controls", "uipics", "graph.png")))
PYFORMS_ICON_EVENTTIMELINE_TIMELINE = QIcon(path(os.path.join("Controls", "uipics", "timeline.png")))
PYFORMS_ICON_EVENTTIMELINE_REFRESH = QIcon(path(os.path.join("Controls", "uipics", "refresh.png")))
PYFORMS_ICON_EVENTTIMELINE_ADD = QIcon(path(os.path.join("Controls", "uipics", "add.png")))
PYFORMS_ICON_EVENTTIMELINE_REMOVE = QIcon(path(os.path.join("Controls", "uipics", "remove.png")))

PYFORMS_ICON_FILE_OPEN = QIcon()


import cv2
import math
import numpy as np
import logging

logger = logging.getLogger(__name__)

def points_distance(p1, p2):
    return  math.hypot(p2[0]-p1[0], p2[1]-p1[1])

def create_rectangle_points(start, end):
    return [ start, (end[0],start[1]), end, (start[0],end[1]) ]

def create_ellipse_points( start, end ):
    width = end[0]-start[0]
    height = end[1]-start[1]
    center = ( start[0] + width/2, start[1] + height/2 )

    distance = points_distance(start, end )
    nPoints = distance / 30
    if nPoints<8:nPoints = 8.0

    points = []
    for angleR in np.arange(0, math.pi*2, math.pi/nPoints):
        x = int(round(center[0] + width/2 * np.cos(angleR) ))
        y = int(round(center[1] + height/2 * np.sin(angleR)))
        points.append( ( x,y) )
    return points

def get_intersection_point_distance(test_point, point1, point2):
    p1 = np.float32(point1)
    p2 = np.float32(point2)
    p3 = np.float32(test_point)
    dist = np.linalg.norm(np.cross(p2-p1, p1-p3))/np.linalg.norm(p2-p1)
    return dist


class ROISelectionWin(object):

    def __init__(self):
        self._selected_point = None # point used to edit the polygon vertice
        self._start_point = None # square starting point
        self._end_point = None # square ending point

    def circlebtn_click_evt(self):
        if self._circlebtn.checked:
            self._roi += ['[]']
            self._roi.selected_row_index = len(self._roi)-1
            self._circlebtn.checked = True

    def polybtn_click_evt(self):
        if self._polybtn.checked:
            self._roi += ['[]']
            self._roi.selected_row_index = len(self._roi)-1
            self._polybtn.checked = True
            
    def create_mask(self, height, width):
        """
        Create a mask based on the selected ROIs
        """
        mask = np.zeros((height, width), dtype=np.uint8)

        if len(self._roi)>0:
            for row in self._roi.value:
                if isinstance(row, str):
                    points = eval( row )
                elif isinstance(row[0], str):
                    points = eval(row[0])
                else:
                    points = row
                if len(points)<3: continue
                mask = cv2.fillPoly(mask, [np.array(points,np.int32)], (255,255,255))
        else:
            mask = mask + 255
        return mask

    def roi_selection_changed_evt(self):
        """
        Function called when the selected polygon change
        """
        self._polybtn.checked = False
        self._circlebtn.checked = False

    def remove_roi(self):
        self._roi -= -1
        self._roi.selected_row_index = None # No poly is selected
        self._player.refresh()

    def draw_rois(self, frame):
        """
        Draw the ROIs lines in the frame.
        """
        # Draw polygons
        try:
            index = self._roi.selected_row_index

            for row_index, row in enumerate(self._roi.value):
                points = eval( row[0] )

                if len(points)>3:
                    cv2.polylines(frame, [np.array(points ,np.int32)], True, (0,255,0), 2, lineType=cv2.LINE_AA)
                else:
                    # draw vertices
                    for i, point in enumerate( points ):
                        point = tuple(point)
                        if self._selected_point == i and index==row_index:
                            cv2.circle(frame, point, 4, (0,0,255), 2)
                        else:
                            cv2.circle(frame, point, 4, (0,255,0), 2)

                if index == row_index and self._selected_point is not None and len(points)>self._selected_point:
                    point = tuple(points[self._selected_point])
                    cv2.circle(frame, point, 4, (0, 0, 255), 2)

        except Exception as e:
            logger.debug(str(e), exc_info=True)

        # Draw the polygons in edition
        if self._start_point and self._end_point:

            if self._rectbtn.checked:
                cv2.rectangle(frame, self._start_point, self._end_point, (233,44,44), 1 )

        return frame



    def select_point(self,x, y):
        """
        select a point from the selected polygon

        :param x: X coordenate
        :param y: Y coordenate
        :return: Index of the selected point in the polygon
        """
        index = self._roi.selected_row_index

        # if a polygon is selected
        if index is not None:
            try:
                coord  = x, y
                points = eval(self._roi.value[index][0])
                for i, point in enumerate(points):
                    if points_distance( coord, point ) <= 5:
                        self._selected_point = i
                        return
                self._selected_point = None
            except:
                pass



    def on_player_double_click_in_video_window(self, event, x, y):
        """
        Event called when the mouse double click.
        This event is used to create vertices on polygons edges.
        :param event: Mouse event
        :param x: X coordenate
        :param y: Y coordenate
        """
        mouse     = int(x), int(y)
        distances = []

        index = self._roi.selected_row_index

        # if a polygon is selected
        if index is not None:
            try:
                points   = list(eval(self._roi.value[index][0]))
                n_points = len(points)
                for point_index, point in enumerate( points ):
                    next_point = points[ (point_index+1) % n_points ]
                    distance = get_intersection_point_distance(mouse, point, next_point )

                    if distance<=5:
                        vector = next_point[0]-point[0], next_point[1]-point[1]
                        center = point[0]+vector[0]/2,point[1]+vector[1]/2
                        radius = points_distance(center, point)
                        mouse_distance = points_distance(center, mouse)
                        if mouse_distance<radius:
                            distances.append( (distance, point_index) )
            except:
                pass

            if len(distances)>0:
                distances = sorted(distances, key=lambda x: x[0])
                point_index = distances[0][1]
                points.insert( point_index + 1, mouse )

                self._roi.set_value(0, index, str(points)[1:-1])

                self._selected_point = point_index + 1

                if not self._player.is_playing: self._player.refresh()


    def on_player_click_in_video_window(self, event, x, y):
        """
        Called when the mouse click in the player
        If the poly button is active it should add the clicked point to the active polygon
        :param event: Mouse event
        :param x: X coordinate
        :param y: Y coordinate
        """
        mouse = int(x), int(y)
        self._selected_point = None

        if not self._rectbtn.checked and not self._polybtn.checked and not self._circlebtn.checked:
            self.select_point( *mouse )

        if self._polybtn.checked or self._circlebtn.checked:
            index = self._roi.selected_row_index
            if index is not None:
                points = list(eval(self._roi.value[index][0]))
                points.append( mouse )
                if len(points)>=5 and self._circlebtn.checked:
                    poly = cv2.fitEllipse(np.array(points,np.int32))
                    center, axis, angle = cv2.fitEllipse(np.array(points,np.int32))
                    center = int(round(center[0])), int(round(center[1]))
                    axis   = int(round(axis[0]/2.0)), int(round(axis[1]/2.0))
                    angle  = int(round(angle))
                    points = cv2.ellipse2Poly(center, axis, angle, 0, 360, 5)
                    self._roi.set_value(0, index, str(points.tolist())[1:-1] + ',')
                    self._circlebtn.checked=False
                else:
                    self._roi.set_value(0, index, str(points)[1:-1]+',')

    def on_player_drag_in_video_window(self, start_point, end_point):
        """
        Called when the mouse drag start
        :param start_point: Top left point
        :param end_point: Bottom right point
        """

        self._start_point = int(start_point[0]), int(start_point[1])
        self._end_point   = int(end_point[0]), int(end_point[1])

        if self._selected_point!=None:
            index = self._roi.selected_row_index
            if index is not None:
                try:
                    points = list(eval(self._roi.value[index][0]))
                    points[self._selected_point] = self._end_point
                    self._roi.set_value(0, index, str(points)[1:-1] + ',')
                except Exception as e:
                    print(e)

        # Refresh the image if the video is not playing
        if not self._player.is_playing:
            self._player.refresh()

    def on_player_end_drag_in_video_window(self, start_point, end_point):
        """
        Called when the mouse drag ends
        :param start_point: Top left point
        :param end_point: Bottom right point
        """
        # Draw the rectangle
        if self._rectbtn.checked:

            self._start_point = int(start_point[0]), int(start_point[1])
            self._end_point = int(end_point[0]), int(end_point[1])
            points = create_rectangle_points(self._start_point, self._end_point)
            self._roi += [str(points)[1:-1]]
            self._start_point    = None
            self._end_point      = None
            self._rectbtn.checked = False


        # Refresh the image if the video is not playing
        if not self._player.is_playing:
            self._player.refresh()


#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlList

"""
import logging
import os


from AnyQt           import QtCore, uic
from AnyQt.QtWidgets import QTableWidgetItem, QWidget, QAbstractItemView, QTableWidgetSelectionRange
from AnyQt.QtGui     import QIcon

from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.controls.control_base import ControlBase

logger = logging.getLogger(__name__)


class ControlList(ControlBase, QWidget):
    """ This class represents a wrapper to the table widget
        It allows to implement a list view
    """

    CELL_VALUE_BEFORE_CHANGE = None  # store value when cell is double clicked

    def __init__(self, *args, **kwargs):
        QWidget.__init__(self)

        self._height = kwargs.get('height', None)

        self._plusFunction  = kwargs.get('add_function', None)
        self._minusFunction = kwargs.get('remove_function', None)
        ControlBase.__init__(self, *args, **kwargs)

        self.autoscroll         = kwargs.get('autoscroll',          True)
        self.resizecolumns      = kwargs.get('resizecolumns',       True)
        self.select_entire_row  = kwargs.get('select_entire_row',   False)
        self.horizontal_headers = kwargs.get('horizontal_headers',  None)


        self.item_selection_changed_event   = kwargs.get('item_selection_changed_event', self.item_selection_changed_event)
        self.data_changed_event             = kwargs.get('data_changed_event', self.data_changed_event)
        self.item_selection_changed_event   = kwargs.get('item_selection_changed_event', self.item_selection_changed_event)
        self.current_cell_changed_event     = kwargs.get('current_cell_changed_event', self.current_cell_changed_event)
        self.current_item_changed_event     = kwargs.get('current_item_changed_event', self.current_item_changed_event)
        self.cell_double_clicked_event      = kwargs.get('cell_double_clicked_event', self.cell_double_clicked_event)

    ##########################################################################
    ############ FUNCTIONS ###################################################
    ##########################################################################

    def init_form(self):
        plusFunction  = self._plusFunction
        minusFunction = self._minusFunction

        # Get the current path of the file
        rootPath = os.path.dirname(__file__)
        # Load the UI for the self instance
        uic.loadUi(os.path.join(rootPath, "list.ui"), self)

        if self._height:
            self.height = self._height

        self.label = self._label

        self.tableWidget.currentCellChanged.connect(self.tableWidgetCellChanged)
        self.tableWidget.currentItemChanged.connect(self.tableWidgetItemChanged)
        self.tableWidget.itemSelectionChanged.connect(self.tableWidgetItemSelectionChanged)
        self.tableWidget.cellDoubleClicked.connect(self.tableWidgetCellDoubleClicked)
        self.tableWidget.model().dataChanged.connect(self._dataChangedEvent)


        self.tableWidget.horizontalHeader().setVisible(False)

        if plusFunction is None and minusFunction is None:
            self.plusButton.hide()
            self.minusButton.hide()
        elif plusFunction is None:
            self.plusButton.hide()
            self.minusButton.pressed.connect(minusFunction)
        elif minusFunction is None:
            self.minusButton.hide()
            self.plusButton.pressed.connect(plusFunction)
        else:
            self.plusButton.pressed.connect(plusFunction)
            self.minusButton.pressed.connect(minusFunction)

        
        super(ControlList, self).init_form()

    def __repr__(self):
        return "ControlList " + str(self._value)

    def clear(self, headers=False):

        for row in range(self.rows_count):
            columns = []
            for column in range(self.columns_count):
                v = self.get_value(column, row)
                if isinstance(v, BaseWidget):
                    v.destroy()

        if headers:
            self.tableWidget.clear()
            self.tableWidget.setColumnCount(3)
            self.tableWidget.setRowCount(0)
        else:
            self.tableWidget.clearContents()
            self.tableWidget.setRowCount(0)

    def save_form(self, data, path=None):
        if self.value:
            rows = []
            for row in range(self.rows_count):
                columns = []
                for column in range(self.columns_count):
                    v = self.get_value(column, row)
                    if isinstance(v, BaseWidget):
                        columns.append(v.save_form({}, path))
                    else:
                        columns.append(str(v))
                rows.append(columns)
            data['value'] = rows
        return data

    def load_form(self, data, path=None):
        if self.value:
            rows = data['value']
            for row in range(len(rows)):
                for column in range(len(rows[row])):
                    v = self.get_value(column, row)
                    if isinstance(v, BaseWidget):
                        v.load_form(rows[row][column], path)
                    else:
                        self.set_value(column, row, rows[row][column])
        elif 'value' in data.keys():
            self.value = data['value']

    def __add__(self, other):
        row_index = self.tableWidget.rowCount()

        self.tableWidget.insertRow(row_index)

        #increase the number of columns if necessary ####
        if self.tableWidget.currentColumn() < len(other):
            self.tableWidget.setColumnCount(len(other))
        #################################################

        for column, e in enumerate(other): self.set_value(column, row_index, e)

        #Auto resize the columns if the flag is activated ################
        if self.resizecolumns:
            self.tableWidget.resizeColumnsToContents()
        ##################################################################

        # auto scroll the list to the new inserted item ##################
        if self.autoscroll:
            self.tableWidget.scrollToItem( self.get_cell(0,row_index) )
        ##################################################################
        return self

    def __sub__(self, other):

        if isinstance(other, int):
            if other < 0:
                indexToRemove = self.tableWidget.currentRow()
            else:
                indexToRemove = other

            self.tableWidget.removeRow(indexToRemove)
        return self

    def set_value(self, column, row, value):
        if isinstance(value, QWidget):
            self.tableWidget.setCellWidget(row, column, value)
            value.show()
            self.tableWidget.setRowHeight(row, value.height())
        elif isinstance(value, ControlBase):
            self.tableWidget.setCellWidget(row, column, value.form)
            value.show()
            self.tableWidget.setRowHeight(row, value.form.height())
        else:
            args = [value] if not hasattr(value, 'icon') else [QIcon(value.icon), value]
            item = QTableWidgetItem()
            item.setData(QtCore.Qt.EditRole, *args)
            self.tableWidget.setItem(row, column, item)

    def get_value(self, column, row):
        try:
            return str(self.tableWidget.item(row, column).text())
        except AttributeError as err:
            return self.tableWidget.cellWidget(row, column)
        except AttributeError as err:
            return ''

    def resize_rows_contents(self):
        self.tableWidget.resizeRowsToContents()

    def get_currentrow_value(self):
        currentRow = self.tableWidget.currentRow()
        if not currentRow < 0:
            return self.value[currentRow]
        else:
            return []

    def get_cell(self, column, row):
        return self.tableWidget.item(row, column)

    def set_sorting_enabled(self, value):
        """
        Enable or disable columns sorting
        
        :param bool value: True to enable sorting, False otherwise 
        """
        self.tableWidget.setSortingEnabled(value)

    ##########################################################################
    ############ EVENTS ######################################################
    ##########################################################################

    def data_changed_event(self, row, col, item):
        pass

    def item_selection_changed_event(self):
        pass

    def current_cell_changed_event(self, next_row, next_col, previous_row, previous_col):
        pass

    def current_item_changed_event(self, current, previous):
        pass

    def cell_double_clicked_event(self, row, column):
        pass

    ##########################################################################
    ############ PROPERTIES ##################################################
    ##########################################################################

    @property
    def height(self):
        return self._height
    @height.setter
    def height(self, value):
        self._height = value
        self.setMaximumHeight(value)

    @property
    def horizontal_headers(self):
        return self._horizontalHeaders

    @horizontal_headers.setter
    def horizontal_headers(self, horizontal_headers):
        """Set horizontal headers in the table list."""
        if horizontal_headers is None:
            self._horizontalHeaders = horizontal_headers
            self.tableWidget.horizontalHeader().setVisible(False)
            return


        self._horizontalHeaders = horizontal_headers

        self.tableWidget.setColumnCount(len(horizontal_headers))
        self.tableWidget.horizontalHeader().setVisible(True)

        for idx, header in enumerate(horizontal_headers):
            item = QTableWidgetItem()
            item.setText(header)
            self.tableWidget.setHorizontalHeaderItem(idx, item)

    @property
    def word_wrap(self):
        return self.tableWidget.wordWrap()

    @word_wrap.setter
    def word_wrap(self, value):
        self.tableWidget.setWordWrap(value)

    @property
    def readonly(self):
        return self.tableWidget.editTriggers()

    @readonly.setter
    def readonly(self, value):
        if value:
            self.tableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        else:
            self.tableWidget.setEditTriggers(QAbstractItemView.AllEditTriggers)

    @property
    def select_entire_row(self):
        return self.tableWidget.selectionBehavior()

    @select_entire_row.setter
    def select_entire_row(self, value):
        if value:
            self.tableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)
        else:
            self.tableWidget.setSelectionBehavior(QAbstractItemView.SelectItems)

    @property
    def rows_count(self):
        return self.tableWidget.rowCount()

    @property
    def columns_count(self):
        return self.tableWidget.columnCount()

    def __len__(self):
        return self.rows_count

    @property
    def value(self):
        if hasattr(self, 'tableWidget'):
            results = []

            for row in range(self.tableWidget.rowCount()):
                r = []
                for col in range(self.tableWidget.columnCount()):
                    try:
                        r.append(self.get_value(col, row))
                    except Exception as err:
                        logger.debug(str(err))
                        r.append("")
                results.append(r)
            return results
        return self._value

    @value.setter
    def value(self, value):
        self.clear()
        for row in value: self += row

    # TODO: implement += on self.value? I want to add a list of tuples to
    # self.value

    @property
    def selected_rows_indexes(self):
        result = []
        for index in self.tableWidget.selectedIndexes():
            result.append(index.row())
        return list(set(result))

    @property
    def selected_row_index(self):
        indexes = self.selected_rows_indexes
        if len(indexes) > 0:
            return indexes[0]
        else:
            return None

    @selected_row_index.setter
    def selected_row_index(self, row):
        self.tableWidget.setRangeSelected(
            QTableWidgetSelectionRange(0, 0, len(self)-1, 0),
            False
        )

        if row is not None and row >= 0:
            self.tableWidget.setRangeSelected(
                QTableWidgetSelectionRange(row, 0, row, 0),
                True
            )

    @property
    def label(self):
        return self.labelWidget.getText()

    @label.setter
    def label(self, value):
        if value.strip() != '':
            self.labelWidget.setText(value)
        else:
            self.labelWidget.hide()

    @property
    def form(self):
        return self

    @property
    def icon_size(self):
        return self.tableWidget.iconSize()

    @icon_size.setter
    def icon_size(self, value):
        if isinstance(value, (tuple, list)):
            self.tableWidget.setIconSize(QtCore.QSize(*value))
        else:
            self.tableWidget.setIconSize(QtCore.QSize(value, value))

    @property
    def autoscroll(self): return self._autoscroll
    @autoscroll.setter
    def autoscroll(self, value): self._autoscroll = value

    @property
    def resizecolumns(self): return self._resizecolumns
    @resizecolumns.setter
    def resizecolumns(self, value): self._resizecolumns = value


    ##########################################################################
    ############ PRIVATE FUNCTIONS ###########################################
    ##########################################################################

    def _dataChangedEvent(self, item):
        self.data_changed_event(item.row(), item.column(), self.tableWidget.model().data(item))
        self.changed_event()

    def tableWidgetCellChanged(self, nextRow, nextCol, previousRow,
                               previousCol):
        self.current_cell_changed_event(nextRow, nextCol, previousRow, previousCol)
        self.changed_event()

    def tableWidgetItemChanged(self, current, previous):
        self.current_item_changed_event(current, previous)
        self.changed_event()

    def tableWidgetItemSelectionChanged(self):
        self.item_selection_changed_event()

    def tableWidgetCellDoubleClicked(self, row, column):
        """
        (From PyQt) This signal is emitted whenever a cell in the table is double clicked.
        The row and column specified is the cell that was double clicked.

        Besides firing this signal, we save the current value, in case the user needs to know the old value.
        :param row:
        :param column:
        :return:
        """
        self.CELL_VALUE_BEFORE_CHANGE = self.get_value(column, row)
        logger.debug("Cell double clicked. Stored value: %s", self.CELL_VALUE_BEFORE_CHANGE)
        self.cell_double_clicked_event(row, column)

    def empty_signal(self, *args, **kwargs):
        """
        Use this function if you want to disconnect a signal temporarily
        """
        pass



# !/usr/bin/python
# -*- coding: utf-8 -*-

from AnyQt.QtGui import QColor
from AnyQt 		 import QtCore

class Pointer(object):
	"""
	
	"""

	def __init__(self, position, parent, scroll):
		"""
		
		:param position: 
		:param parent: 
		:param scroll: 
		"""
		self._position = position
		self._parent = parent
		self._scroll = scroll

	def draw(self, painter, left_shift=0, scale=1):
		"""
		
		:param painter: 
		:param left_shift: 
		:param scale: 
		:return: 
		"""
		x = self.position / scale + left_shift

		painter.setPen(QColor(0, 255, 0))
		painter.setBrush(QColor(0, 255, 0))
		painter.drawLine(x, 8, x, self._parent.height())
		painter.drawEllipse(QtCore.QPoint(x, 8), 5, 5)
		painter.drawText(x + 8, 8 + 4, str(self.position))

	##########################################################################
	#### HELPERS/FUNCTIONS ###################################################
	##########################################################################

	def moveEvent(self):
		"""
		
		"""
		pass

	def collide(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		return (self.position - 5) <= x <= (self.position + 5) and 3 <= y <= 11

	def canSlideBegin(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:rtype: bool 
		"""
		return False

	def canSlideEnd(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:rtype: bool 
		"""
		return False

	def move(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		if (self._position - x) >= 0 and (self._position - x) <= (self._parent.width()):
			self._position += x
			self.moveEvent()

	##########################################################################
	#### PROPERTIES ##########################################################
	##########################################################################

	@property
	def position(self): return self._position

	@position.setter
	def position(self, value):
		self._position = value
		self.moveEvent()

	@property
	def frame(self): return self._position



import csv, sys
from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.controls.control_progress import ControlProgress
from pyforms_gui.controls.control_button import ControlButton
from pyforms_gui.controls.control_list import ControlList
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_slider import ControlSlider
from pyforms_gui.controls.control_textarea import ControlTextArea
from pyforms_gui.controls.control_text import ControlText
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_label import ControlLabel
import traceback
import time
import datetime

from confapp import conf
from AnyQt.QtWidgets import QMessageBox



class GraphsEventsGenerator(BaseWidget):

    def __init__(self, timeline=None):
        super(GraphsEventsGenerator, self).__init__('Apply a function to the graph values', parent_win=timeline)
        self.setContentsMargins(10, 10, 10, 10)
        self._timeline = timeline


        # Definition of the forms fields
        self._graphs_list = ControlList('Graphs list (try double click)', readonly=False, select_entire_row=True)
        self._equation    = ControlTextArea('Equation')
        self._graphname   = ControlText('Graph name')
        self._genevts_btn = ControlButton('Generate graph')

        
        self._formset = [
            (
                ['_graphs_list'],
                '||', 
                [
                    '_graphname',
                    '_equation',
                    '_genevts_btn',
                ]
            ),
        ]

        self._graphs_list.cell_double_clicked_event = self.__cell_double_clicked_evt 
        self._graphs_list.readonly = True
        self._graphs_list.select_entire_row = True
        self._genevts_btn.value = self.__generage_events_evt



    def __add__(self, other):
        self._graphs_list += [other.name]
        return self

    def __sub__(self, other):
        self._graphs_list -= other
        return self

    def rename_graph(self, graph_index, newname):
        self._graphs_list.set_value(graph_index, 0, newname)

    @property
    def graphs(self):
        return self._timeline._graphs

    def show(self):
        super(GraphsEventsGenerator, self).show()

        if len(self._graphname.value.strip())==0:
            self._graphname.value = "generated-graph-{0}".format( len(self._timeline.graphs)) 



    def __cell_double_clicked_evt(self, row, column):
        if len(self._equation.value.strip())==0:
            self._equation.value += '[{0}]'.format( self._graphs_list.value[row][column] )
        else:
            self._equation.value += ' and [{0}]'.format( self._graphs_list.value[row][column] )


    def __generage_events_evt(self):
        if len(self._graphname.value.strip())==0: 
            QMessageBox.warning(
                self, "Attention!", 'The graph name cannot be empty')
            return

        if len(self._equation.value.strip())==0: 
            QMessageBox.warning(
                self, "Attention!", 'The equation cannot be empty')
            return 

        max_frame = 0
        equation  = self._equation.value
        for i, values in enumerate(self._graphs_list.value):
            graphname = '[{0}]'.format(values[0])

            if graphname in equation: max_frame = max(max_frame, len(self.graphs[i]))

            equation  = equation.replace(graphname, 'graphs[{0}][i]'.format(i) )
        
        graphs = self.graphs
        data   = []
        try:
            for i in range(max_frame):

                try:
                    value = eval(equation)
                except:
                    value = None

                data.append( (i, value) )
            self._timeline.add_graph(self._graphname.value, data)

            self._graphname.value = "generated-graph-{0}".format( len(self._timeline.graphs)) 
        except Exception as e:
            traceback.print_exc()
            QMessageBox.warning( self, "Error!", str(e) )

    
##################################################################################################################
##################################################################################################################
##################################################################################################################

# Execute the application
if __name__ == "__main__":
    import pyforms
    pyforms.start_app(GraphsEventsGenerator, geometry=(0, 0, 600, 400))



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = '4.0'
__maintainer__  = ["Ricardo Ribeiro"]
__email__       = ["ricardojvr@gmail.com"]
__status__      = "Production"


from confapp import conf

try:
    import local_settings
    conf += local_settings
except:
    pass

conf += 'pyforms.settings'




if conf.PYFORMS_MODE == 'GUI':

    from pyforms_gui.appmanager import start_app

elif conf.PYFORMS_MODE == 'TERMINAL':

    from pyforms_terminal.appmanager import start_app


#!/usr/bifn/python
# -*- coding: utf-8 -*-

from confapp import conf

from AnyQt                              import _api
from AnyQt.QtWidgets                    import QWidget, QVBoxLayout
from pyforms_gui.controls.control_base  import ControlBase
from pyforms_gui.basewidget             import BaseWidget


class ControlEmptyWidget(ControlBase, QWidget):
    def __init__(self, *args, **kwargs):
        QWidget.__init__(self)
        layout = QVBoxLayout()

        if _api.USED_API == _api.QT_API_PYQT5:
            layout.setContentsMargins(0,0,0,0)
        elif _api.USED_API == _api.QT_API_PYQT4:
            layout.setMargin(0)


        self.form.setLayout(layout)

        ControlBase.__init__(self, *args, **kwargs)
        self.value = kwargs.get('default', None)


    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def value(self):
        return ControlBase.value.fget(self)

    @value.setter
    def value(self, value):
        ControlBase.value.fset(self, value)
        self.__clear_layout()

        if value is None or value == '':  return

        if isinstance(self._value, list):
            for w in self._value:
                if w != None and w != "": self.form.layout().removeWidget(w.form)

        if isinstance(value, list):
            for w in value:
                self.form.layout().addWidget(w.form)
                
                # The init_form should be called only for the BaseWidget
                if isinstance(w, BaseWidget) and not w._formLoaded:
                    w.init_form()
                if not hasattr(w, 'about_to_be_shown'):
                    w.about_to_be_shown = True
                    w.show()
                    del w.about_to_be_shown
        else:
            self.form.layout().addWidget(value.form)

            # The init_form should be called only for the BaseWidget
            if isinstance(value, BaseWidget) and not value._formLoaded:
                value.init_form()

            if not hasattr(value, 'about_to_be_shown'):
                value.about_to_be_shown = True
                value.show()
                del value.about_to_be_shown
        
        
    @property
    def form(self):
        return self

    def save_form(self, data, path=None):
        if self.value is not None and self.value != '':
            data['value'] = {}
            self.value.save_form(data['value'], path)
        return data

    def load_form(self, data, path=None):
        if 'value' in data and self.value is not None and self.value != '':
            self.value.load_form(data['value'], path)

    def __clear_layout(self):
        if self.form.layout() is not None:
            old_layout = self.form.layout()
            for i in reversed(range(old_layout.count())):
                old_layout.itemAt(i).widget().setParent(None)

    def show(self):
        """
        Show the control
        """
        QWidget.show(self)

    def hide(self):
        """
        Hide the control
        """
        QWidget.hide(self)



from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlMatplotlib

class GraphAreaWin(BaseWidget):

    def __init__(self, *args, **kwargs):
        super().__init__(title='Area graph')

        self._graph = ControlMatplotlib('Blobs area', toolbar=False, enabled=False)

        self.formset = ['_graph']

    @property
    def enabled(self):
        return self._graph.enabled

    @enabled.setter
    def enabled(self, value):
        self._graph.enabled = value

    @property
    def on_draw(self):
        return self._graph.on_draw

    @on_draw.setter
    def on_draw(self, value):
        self._graph.on_draw = value

    def draw(self):
        self._graph.draw()


#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"


import os
import subprocess



class BioradWriter:

	"""
		Encapsulation of ImageJ's (http://rsb.info.nih.gov/ij/) 'Biorad Writer' plugin to enable
		the transformation of TIFF images into PIC format directly from the command line.
	"""

	# Configuration parameters
	# NB: Keep the trailing slashes
	imageJDir = '/home/admin/imagej/'
	javaDir = imageJDir + 'jdk/bin/'
	bioradWriterConfigFile = 'biorad_writer_config.txt'

	@staticmethod
	def __update_config_file(outputFileName):
		try:
			with open(BioradWriter.imageJDir + BioradWriter.bioradWriterConfigFile, 'w') as f:
				f.write(os.getcwd() + "/output/\n");
				f.write(outputFileName);		
	    	except (IOError) as e:
	    		raise e

	@staticmethod
	def convert(inputFileName, outputFileName='', 
		imageJDir='/home/admin/imagej/', 
		javaDir='/home/admin/imagej/jdk/bin/', 
		bioradWriterConfigFile='biorad_writer_config.txt' ):
		
		if not outputFileName:
			outputFileName = os.path.splitext(inputFileName)[0] + '.PIC'

		BioradWriter.__update_config_file(outputFileName)
		execStr = 'cd ' + BioradWriter.imageJDir + ";"
		execStr += BioradWriter.javaDir + 'java -cp ' + BioradWriter.imageJDir + 'ij.jar ij.ImageJ -batch ' + BioradWriter.imageJDir + 'macro.ijm ' + inputFileName + ';'
		execStr += 'cd -'
		subprocess.call(execStr, shell=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os, fnmatch, re

version = ''
license = ''
with open('pyforms_gui/__init__.py', 'r') as fd:
    content = fd.read()
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', content, re.MULTILINE).group(1)

    license = re.search(
        r'^__license__\s*=\s*[\'"]([^\'"]*)[\'"]', content, re.MULTILINE).group(1)

if not version: raise RuntimeError('Cannot find version information')
if not license: raise RuntimeError('Cannot find license information')

with open('README.md', 'r') as fd:
    long_description = fd.read()

setup(
    name='PyForms GUI',
    version=version,
    description="""Pyforms is a Python framework to develop GUI applications based on pyqt""",
    author='Ricardo Ribeiro, Luís Teixeira',
    author_email='ricardojvr@gmail.com, micboucinha@gmail.com',
    license=license,
    url='https://github.com/UmSenhorQualquer/pyforms-gui',
    
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    python_requires='>=3',
    install_requires=[
        'anyqt',
        "pyqt5; python_version>='3'",
        "PyQtWebEngine",
        'pyopengl',
        "QScintilla; python_version>='3'",
        'visvis',
        'matplotlib',
        'python-dateutil',
        'numpy',
        'opencv-python==3.4.5.20',
        'confapp',
        'natsort'
    ],
    packages=find_packages(),
    package_data={'pyforms_gui': [
        'controls/uipics/*.png',
        'controls/*.ui', 
        'controls/control_player/*.ui',
        'controls/control_event_timeline/*.ui',
        'controls/control_event_timeline/events/*.ui']
    },


    classifiers=[
        'Development Status :: 5 - Production/Stable',
        
        'Topic :: Software Development :: Build Tools',
        
        'Programming Language :: Python :: 3',

        'Environment :: MacOS X',
        'Environment :: X11 Applications :: Qt',
        'Environment :: Win32 (MS Windows)',

        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Unix',

        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        
    ],

    keywords='terminal development pyforms'
)



# !/usr/bin/python
# -*- coding: utf-8 -*-

from AnyQt.QtGui import QColor
from AnyQt import QtCore


class Pointer(object):
	def __init__(self, position, parent):
		"""
        		
		:param position: 
		:param parent: 
		"""
		self._position = position
		self._parent = parent

	def draw(self, painter, showvalues=False, highlight=False):
		"""
		
		:param painter: 
		:param showvalues: 
		:return: 
		"""
		if highlight:
			painter.setPen(QColor(0, 150, 150))
			painter.setBrush(QColor(0, 150, 150))
		else:
			painter.setPen(QColor(0, 255, 0))
			painter.setBrush(QColor(0, 255, 0))


		painter.drawLine(
			self.xposition, 8, self.xposition, self._parent.height())
		painter.drawEllipse(QtCore.QPoint(self.xposition, 8), 5, 5)
		painter.drawText(self.xposition + 8, 8 + 4, str(self._position))

	##########################################################################
	#### HELPERS/FUNCTIONS ###################################################
	##########################################################################

	def moveEvent(self):
		"""
		
		:return: 
		"""
		pass

	def collide(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		return self.begin <= x <= self.end and 0 <= y <= self._parent.TOPTRACK_HEIGHT

	def can_slide_begin(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		return False

	def can_slide_end(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		return False

	def move(self, x, y):
		"""
		
		:param x: 
		:param y: 
		:return: 
		"""
		x = int(round(x / self._parent._scale))
		if (self._position - x) >= 0 and (self._position - x) <= (self._parent.width() / self._parent._scale):
			self._position += x
			self.moveEvent()

	##########################################################################
	#### PROPERTIES ##########################################################
	##########################################################################
	@property
	def xposition(self):
		return self._parent.frame2x(self.position)

	@property
	def position(self):
		return self._position

	@position.setter
	def position(self, value):
		self._position = value
		self.moveEvent()

	@property
	def frame(self): return self._position

	@property
	def begin(self):
		return self._position * self._parent.scale - 5

	@property
	def end(self):
		return self._position * self._parent.scale + 5



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"


import os
import sys
# import importlib
import math
from numpy import *
import zipfile
from subprocess import Popen, PIPE
import subprocess

# THIS DOESN'T WORK BECAUSE CONF IS IMPORTED WITH PYFORMS
# def load_everything_from(module_names):
#     """
#     Load all
#     :param module_names: modules to be imported
#     :type module_names: list of strings
#     """
#     g = globals()
#     for module_name in module_names:
#         m = importlib.import_module(module_name)
#         names = getattr(m, '__all__', None)
#         if names is None:
#             names = [name for name in dir(m) if not name.startswith('_')]
#         for name in names:
#             g[name] = getattr(m, name)

# THIS DOESN'T WORK BECAUSE CONF IS IMPORTED WITH PYFORMS
# def register_settings_module(module):
#     """
#     Append another settings module to PYFORMS_APP_SETTINGS
#     :param module:
#     :type module:
#     """
#     modules = os.getenv('PYFORMS_APP_SETTINGS', '').split(';')
#     modules.append(module)
#     os.environ['PYFORMS_APP_SETTINGS'] = ';'.join(modules)


def getFileInSameDirectory(file, name):
	module_path = os.path.abspath(os.path.dirname(file))
	return os.path.join(module_path, name)

def groupImagesHorizontally( images, color=False ):
	final_width, final_height = sum([ x.shape[1] for x in images ]), max([ x.shape[0] for x in images ])
	if color:
	    final_image = zeros( (final_height, final_width, 3), dtype=uint8 )
	else:
	    final_image = zeros( (final_height, final_width), dtype=uint8 )
	cursor = 0
	for image in images:
	    final_image[0:image.shape[0],cursor:cursor+image.shape[1]] = image
	    cursor += image.shape[1]
	return final_image


def groupImagesVertically( images, color=False ):
	final_width, final_height = max([ x.shape[1] for x in images ]), sum([ x.shape[0] for x in images ])
	if color:
	    final_image = zeros( (final_height, final_width, 3), dtype=uint8 )
	else:
	    final_image = zeros( (final_height, final_width), dtype=uint8 )
	cursor = 0
	for image in images:
	    final_image[cursor:cursor+image.shape[0],0:image.shape[1]] = image
	    cursor += image.shape[0]
	return final_image

def groupImage( images,color=False ):
	himages = []
	for group in images:
	    if isinstance(group, list):
	        himages.append( groupImagesVertically(group,color) )
	    else:
	        himages.append( group )

	return groupImagesHorizontally(himages,color)


def get_object_class_path(obj):
	path = os.path.abspath(sys.modules[obj.__module__].__file__)
	head, tail = os.path.split(path)
	return head


def lin_dist(p0, p1):   return math.sqrt((p0[0] - p1[0])**2 + (p0[1] - p1[1])**2)

def lin_dist3d(p0, p1):   return math.sqrt((p0[0] - p1[0])**2 + (p0[1] - p1[1])**2 + (p0[2] - p1[2])**2)


def zipdir(path, zippath):
    """
    walkfiles = os.walk(path)
    zippath = zipfile.ZipFile(zippath, 'w')
    for root, dirs, files in walkfiles:
        for filename in files: zippath.write(os.path.join(root, filename))
    """
    execStr = ['zip', '-r',zippath, path]
    proc = subprocess.Popen(execStr, stdout=PIPE, stderr=PIPE)
    (output, error) = proc.communicate()
    if error: print ('error: '+ error)
        

def zipfiles(files, zippath):
    zippath = zipfile.ZipFile(zippath, 'w')
    for filename in files: zippath.write( filename)
		
def points_angle(p1, p2): 
    x1, y1 = p1
    x2, y2 = p2
    rads = math.atan2(-(y2-y1),x2-x1)
    rads %= 2*pi
    return rads


def getTranslationMatrix2d(dx, dy):
    """
    Returns a numpy affine transformation matrix for a 2D translation of
    (dx, dy)
    """
    return matrix([[1, 0, dx], [0, 1, dy], [0, 0, 1]])



def combinations( l1, l2 ):
    """
    Make combinations between the 2 lists without repeating
    """
    for i in range(len(l1)):
        yield zip( l1,l2)
        l1.insert(0,l1.pop())


#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlLabel"""

import pyforms_gui.utils.tools as tools
from confapp import conf

from AnyQt import uic, QtCore, QtGui

from pyforms_gui.controls.control_base import ControlBase

__author__ = "Carlos Mão de Ferro"
__copyright__ = ""
__credits__ = "Carlos Mão de Ferro"
__license__ = "MIT"
__version__ = "0.0"
__maintainer__ = ["Ricardo Ribeiro", "Carlos Mão de Ferro"]
__email__ = ["ricardojvr at gmail.com", "cajomferro at gmail.com"]
__status__ = "Development"


class ControlLabel(ControlBase):

    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "label.ui")
        self._form = uic.loadUi(control_path)
        self._form.label.setText(self._label)
        self._selectable = False
        super(ControlLabel, self).init_form()

    def load_form(self, data, path=None): pass

    def save_form(self, data, path=None): pass

    @property
    def selectable(self): return self._selectable

    @selectable.setter
    def selectable(self, value):
        if value:
            self._form.label.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
            self._form.label.setCursor(QtGui.QCursor(QtCore.Qt.IBeamCursor))
        else:
            self._form.label.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
            self._form.label.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))

    @property
    def form(self): return self._form


    @property
    def value(self): return ControlBase.value.fget(self)

    @value.setter
    def value(self, value):
        self._form.label.setText(value)
        ControlBase.value.fset(self, value)




from confapp import conf



if conf.PYFORMS_MODE in ['GUI']:

    from pyforms_gui.allcontrols import *

elif conf.PYFORMS_MODE in ['TERMINAL']:

    from pyforms_terminal.allcontrols import *

elif conf.PYFORMS_MODE in ['WEB']:

    from pyforms_web.allcontrols import *


import csv
from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.allcontrols import ControlText
from pyforms_gui.allcontrols import ControlButton
from pyforms_gui.allcontrols import ControlFile
from pyforms_gui.allcontrols import ControlList
from pyforms_gui.allcontrols import ControlNumber
from pyforms_gui.allcontrols import ControlCombo


class GenericCsvParserDialog(BaseWidget):
    def __init__(self, columns, parent=None):
        super(GenericCsvParserDialog, self).__init__('CSV Choose the columns', parent_win=parent)

        self._filename = None
        self._columns = columns
        self._columns_indexes = []
        self._rownum = 0

        # Definition of the forms fields
        self._filename      = ControlFile('CSV File')
        self._separator     = ControlCombo('Separator', default='auto')
        self._startingrow   = ControlNumber('Starting row', default=0)

        for index, column in enumerate(columns):
            setattr(self, '_col_{0}'.format(index), ControlNumber(column, default=index, minimum=-1, maximum=1000))

        self._filePreview = ControlList('Preview')
        self._loadButton  = ControlButton('Load')

        form_row = ['_separator'] + ['_col_{0}'.format(index) for index, column in enumerate(columns)] + ['_loadButton']

        self._formset = [
            ('_filename', '_startingrow'),
            tuple(form_row),
            '_filePreview'
        ]
        self._separator.changed_event = self.__refreshPreview
        self._filename.changed_event = self.__refreshPreview
        self._startingrow.changed_event = self.__refreshPreview
        self._loadButton.value = self.load

        self._load_event = None

        self._separator.add_item('auto', 'auto')
        self._separator.add_item(';', ';')
        self._separator.add_item(',', ',')
        self._separator.add_item('TAB', '\t')
        self._separator.add_item('Excel', csv.excel)
        self._separator.add_item('Excel TAB', csv.excel_tab)

    @property
    def load_file_event(self):
        return self._load_event

    @load_file_event.setter
    def load_file_event(self, value):
        self._load_event = value

    def __iter__(self):

        if self._filename.value != None and self._filename.value != '':
            csvfile = open(self._filename.value, 'U')
            if self.delimiter in ['auto',  csv.excel,  csv.excel_tab]:
                try:
                    dialect = csv.Sniffer().sniff(csvfile.read(1024))
                    csvfile.seek(0)
                    self._spamreader = csv.reader(csvfile, dialect)
                except:
                    self._spamreader = None
                    return self
            else:
                self._spamreader = csv.reader(csvfile, delimiter=self.delimiter)
            for i in range(int(self._startingrow.value)): next(self._spamreader, None)  # skip the headers
        else:
            self._spamreader = None

        return self

    # For compatibility with python 3
    def __next__(self):
        return self.next()

    def next(self):
        if self._spamreader != None:
            row = next(self._spamreader)
            res = []
            for col in self._columns_indexes:
                if col == -1:
                   res.append(self._rownum)
                else:
                    if row[col].lower() == 'nan':
                        res.append('None')
                    else:
                        res.append(row[col])

            self._rownum +=1

            return res
        else:
            raise StopIteration()

    def __refreshPreview(self):
        if self._filename.value != None and self._filename.value != '':
            with open(self._filename.value, 'U') as csvfile:
                if self.delimiter in ['auto',  csv.excel,  csv.excel_tab]:
                    try:
                        dialect = csv.Sniffer().sniff(csvfile.read(1024))
                        csvfile.seek(0)
                        spamreader = csv.reader(csvfile, dialect)
                    except:
                        self.message('Error when reading the file.')
                        return
                else:
                    spamreader = csv.reader(csvfile, delimiter=self.delimiter)

                for i in range(int(self._startingrow.value)): next(spamreader, None)  # skip the headers
                self._filePreview.value = []
                self._filePreview.horizontalHeaders = map(str, range(1000))
                for i, row in enumerate(spamreader):
                    self._filePreview += row
                    if i >= 10:
                        break

    @property
    def delimiter(self):
        return self._separator.value

    def load(self):
        self.__refreshPreview()
        self._columns_indexes = []
        for index, column in enumerate(self._columns):
            self._columns_indexes.append(int(getattr(self, '_col_{0}'.format(index)).value))
        if self._load_event is not None: self._load_event()


##################################################################################################################
##################################################################################################################
##################################################################################################################

# Execute the application
if __name__ == "__main__":
    import pyforms

    pyforms.start_app(GenericCsvParserDialog, geometry=(0, 0, 600, 400))


#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp import conf

import pyforms_gui.utils.tools as tools

from AnyQt           import uic, QtCore
from AnyQt.QtWidgets import QListWidgetItem

from pyforms_gui.controls.control_base import ControlBase


class ControlCheckBoxList(ControlBase):

    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "tree.ui")
        self._form = uic.loadUi(control_path)

        self._form.label.setText(self._label)

        self._form.listWidget.itemChanged.connect(self.item_changed)

        self._form.listWidget.itemSelectionChanged.connect(self.__itemSelectionChanged)

        if self.help: self.form.setToolTip(self.help)
        super(ControlCheckBoxList, self).init_form()




    def save_form(self, data={}, path=None):
        results = []

        for row in range(self.count):
            item = self._form.listWidget.item(row)
            if item != None and item.checkState() == QtCore.Qt.Checked:
                results.append(item.text())
        data['selected'] = results

        return data

    def load_form(self, data, path=None):
        results = data['selected']

        for row in range(self.count):
            item = self._form.listWidget.item(row)
            if item != None and  item.text() in results:
                item.setCheckState(QtCore.Qt.Checked)
            else:
                item.setCheckState(QtCore.Qt.Unchecked)



    def item_changed(self, item):
        self.changed_event()

    def __add__(self, val):
        if isinstance(val, (tuple, list)):
            item = QListWidgetItem(str(val[0]))
            item.value = val[0]
            if val[1]:
                item.setCheckState(QtCore.Qt.Checked)
            else:
                item.setCheckState(QtCore.Qt.Unchecked)
        else:
            item = QListWidgetItem(str(val))
            item.value = val

        self._form.listWidget.addItem(item)
        return self

    def __sub__(self, other):
        if isinstance(other, int):
            if other < 0:
                indexToRemove = self._form.listWidget.currentRow()
            else:
                indexToRemove = other
            self._form.listWidget.takeItem(indexToRemove)
        else:
            for row in range(self.count):
                item = self._form.listWidget.item(row)
                if item != None and hasattr(item, 'value') and item.value == other:
                    self._form.listWidget.takeItem(row)
        return self

    def clear(self):
        self._form.listWidget.clear()

    def refresh(self):
        for row in range(self.count):
            item = self._form.listWidget.item(row)
            if hasattr(item, 'value'): item.setText(str(item.value))

    ############################################################################
    ############ Events ########################################################
    ############################################################################

    def __itemSelectionChanged(self):
        self.selection_changed_event()

    def selection_changed_event(self):
        pass

    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def count(self):
        return self._form.listWidget.count()

    @property
    def checked_indexes(self):
        results = []
        for row in range(self.count):
            item = self._form.listWidget.item(row)
            if item != None and item.checkState() == QtCore.Qt.Checked: results.append(row)
        return results

    @property
    def value(self):
        results = []
        for row in range(self.count):
            item = self._form.listWidget.item(row)
            if item != None and item.checkState() == QtCore.Qt.Checked:
                results.append(item.value if hasattr(item, 'value') else str(item.text()))
        return results

    @value.setter
    def value(self, value):
        self.clear()
        for row in value: self += row
        self.changed_event()

    @property
    def selected_row_index(self):
        return self.form.listWidget.currentRow()

    @property
    def items(self):
        results = []
        for row in range(self.count):
            item = self._form.listWidget.item(row)
            results.append(
                [item.value if hasattr(item, 'value') else str(item.text()), item.checkState() == QtCore.Qt.Checked])
        return results



# !/usr/bin/python
# -*- coding: utf-8 -*-

import csv
from pyforms_gui.controls.control_base 					  import ControlBase
from pyforms_gui.controls.control_events_graph.EventsWidget import EventsWidget
from AnyQt.QtWidgets import QWidget, QVBoxLayout, QScrollArea, QScrollBar, QFileDialog
from AnyQt 			 import QtCore, _api

class ControlEventsGraph(ControlBase, QWidget):
	"""
		Timeline events editor
	"""

	def __init__(self, label="", default=0, min=0, max=100, **kwargs):
		"""
		
		:param label: 
		:param default: 
		:param min: 
		:param max: 
		:param kwargs: 
		"""
		QWidget.__init__(self)
		ControlBase.__init__(self, label, default, **kwargs)
		self.add_popup_menu_option('Export to CSV', self.__export)

	def init_form(self):
		vlayout = QVBoxLayout()

		if _api.USED_API == _api.QT_API_PYQT5:
			vlayout.setContentsMargins(0,0,0,0)
		elif _api.USED_API == _api.QT_API_PYQT4:
			vlayout.setMargin(0)

		self.setLayout(vlayout)

		self._scroll = QScrollBar(QtCore.Qt.Horizontal)

		scrollarea = QScrollArea()
		scrollarea.setMinimumHeight(140)
		scrollarea.setWidgetResizable(True)

		self._events_widget = EventsWidget(scroll=self._scroll)
		scrollarea.setWidget(self._events_widget)

		self._scroll.actionTriggered.connect(self.__scroll_changed)

		vlayout.addWidget(scrollarea)  # The timeline widget
		vlayout.addWidget(self._scroll)  # Add scroll

		self._scroll.setMaximum(0)
		self._scroll.setSliderPosition(0)

	##########################################################################
	#### HELPERS/PUBLIC FUNCTIONS ############################################
	##########################################################################

	def add_track(self, title=None):
		"""
		
		:param title: 
		"""
		self._events_widget.add_track(title)

	def add_event(self, begin, end, title='', track=0, color='#FFFF00'):
		"""
		
		:param begin: 
		:param end: 
		:param title: 
		:param track: 
		:param color: 
		:return: 
		"""
		return self._events_widget.add_event(begin, end, title, track, color)

	##########################################################################
	#### EVENTS ##############################################################
	##########################################################################

	def __scroll_changed(self, change): self.repaint()

	def get_export_filename(self): return "untitled.csv"

	def __export(self):
		"""Export annotations to a file."""
		filename, _ = QFileDialog.getSaveFileName(parent=self,
		                                       caption="Export annotations file",
		                                       directory=self.get_export_filename(),
		                                       filter="CSV Files (*.csv)",
		                                       options=QFileDialog.DontUseNativeDialog)
		if filename != '':
			self.export_csv(filename)

	def export_csv(self, filename):
		"""
		Export annotations to a file.
		:param str filename: filename to open 
		"""
		with open(filename, 'wb') as csvfile:
			spamwriter = csv.writer(csvfile, dialect='excel')
			self._events_widget.export_csv(spamwriter)

	def repaint(self):
		"""
		
		"""
		self._events_widget.repaint()

	##########################################################################
	#### PROPERTIES ##########################################################
	##########################################################################
	"""
	Overwrite the changed event from the ControlBase
	"""

	@property
	def changed_event(self): return self._events_widget._pointer.moveEvent

	@changed_event.setter
	def changed_event(self, value): self._events_widget._pointer.moveEvent = value

	@property
	def value(self): return self._events_widget.position

	@value.setter
	def value(self, value): self._events_widget.position = value

	@property
	def form(self): return self

	@property
	def tracks(self): return self._events_widget.tracks

	@property
	def tracks_height(self): return self._events_widget.tracks_height

	@tracks_height.setter
	def tracks_height(self, value): self._events_widget.tracks_height = value

	@property
	def scale(self): return self._events_widget.scale

	@scale.setter
	def scale(self, value): self._events_widget.scale = value



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"

import visvis as vv, numpy as np

from visvis import Point, Pointset
from pyforms_gui.controls.control_base import ControlBase
from AnyQt                            import _api
from AnyQt.QtWidgets                  import QWidget, QVBoxLayout, QSizePolicy


class ControlVisVis(ControlBase):

    def init_form(self):        
        self._form = QWidget()
        layout = QVBoxLayout()
        
        if _api.USED_API == _api.QT_API_PYQT5:
            layout.setContentsMargins(0,0,0,0)
        else:
            layout.setMargin(0)

        self._form.setLayout( layout )
        self._app = vv.use('pyqt5')
        self._app.Create()
        
        Figure = self._app.GetFigureClass()
        self._fig = Figure(self._form)

        policy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        widget = self._fig._widget
        widget.setSizePolicy(policy)

        layout.addWidget(widget)
        super(ControlVisVis, self).init_form()

        
    def refresh(self):
        vv.figure(self._fig.nr)
        #self._app = vv.use()
        self.paint(vv)

        
        
    def paint(self, visvis):
        vv.clf()  
        
        colors = ['r','g','b','c','m','y','k']
        for index, dataset in enumerate(self._value):
            l = visvis.plot(dataset, ms='o', mc=colors[ index % len(colors) ], mw='3', ls='', mew=0 )
            l.alpha = 0.3

        self._a = vv.gca()
        self._a.daspectAuto = True



    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def legend(self):return self._a.legend
    @legend.setter
    def legend(self, value): self._a.legend = value

    @property
    def show_grid(self):return self._a.axis.showGrid
    @show_grid.setter
    def show_grid(self, value): self._a.axis.showGrid = value

    @property
    def title(self):return ''
    @title.setter
    def title(self, value): vv.title(value)

    @property
    def xlabel(self):return self._a.axis.xlabel
    @xlabel.setter
    def xlabel(self, value): self._a.axis.xlabel = value

    @property
    def ylabel(self):return self._a.axis.ylabel
    @ylabel.setter
    def ylabel(self, value): self._a.axis.ylabel = value

    @property
    def zlabel(self):return self._a.axis.zlabel
    @ylabel.setter
    def zlabel(self, value): self._a.axis.zlabel = value

    @property
    def value(self): return None

    @value.setter
    def value(self, value):
        self._value = []
        for dataset in value:
            if len(dataset)>0:
                if isinstance(dataset[0], list) or isinstance(dataset[0], tuple) :
                    self._value.append( Pointset( np.array(dataset) ) )
                else:
                    self._value.append( dataset )

        self.refresh()
        



#!/usr/bin/python
# -*- coding: utf-8 -*-


from confapp import conf

import pyforms_gui.utils.tools as tools

from AnyQt import uic

from pyforms_gui.controls.control_base import ControlBase


class ControlNumber(ControlBase):
    def __init__(self, *args, **kwargs):
        """
        :param int minimum: Minimum value.
        :param int maximum: Maximum value.
        :param float default: Set the value. Default = 0.
        :param int decimals: Decimals precision.
        :param float step: Step jump value.
        """
        self._min  = kwargs.get('minimum', 0)
        self._max  = kwargs.get('maximum', 100)
        if 'default' not in kwargs: kwargs['default'] = 0
        ControlBase.__init__(self, *args, **kwargs)
        self.decimals = kwargs.get('decimals', 0)
        self.step     = kwargs.get('step', 1)
        
    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "number.ui")
        self._form = uic.loadUi(control_path)
        self.min = self._min
        self.max = self._max
        self.label = self._label
        self.form.label.setAccessibleName('ControlNumber-label')
        self.form.spinBox.valueChanged.connect(self.update_event)

        self._update_slider = True
        self.form.spinBox.setValue(self._value)
        del self._update_slider

        super(ControlNumber, self).init_form()

        
    def update_event(self, value):
        if not hasattr(self, '_update_slider'):
            self.value = value

    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def label(self): return self.form.label.text()

    @label.setter
    def label(self, value): self.form.label.setText(value)

    @property
    def value(self):
        return self._value
        return self.form.spinBox.value()

    @value.setter
    def value(self, value):
        if value != self.value:
            self._update_slider = True
            self.form.spinBox.setValue(value)
            del self._update_slider

        ControlBase.value.fset(self, self.form.spinBox.value())

    @property
    def min(self): return self.form.spinBox.minimum()

    @min.setter
    def min(self, value): self.form.spinBox.setMinimum(value)

    @property
    def max(self): return self.form.spinBox.maximum()

    @max.setter
    def max(self, value): self.form.spinBox.setMaximum(value)

    @property
    def decimals(self): return self.form.spinBox.decimals()

    @decimals.setter
    def decimals(self, value): self.form.spinBox.setDecimals(value)

    @property
    def step(self): return self.form.spinBox.singleStep()

    @step.setter
    def step(self, value): self.form.spinBox.setSingleStep(value)   



from confapp import conf


if conf.PYFORMS_MODE in ['GUI']:

    from pyforms_gui.basewidget import BaseWidget
    from pyforms_gui.organizers import vsplitter, hsplitter, segment, no_columns

elif conf.PYFORMS_MODE in ['TERMINAL']:

    from pyforms_terminal.basewidget import BaseWidget
    no_columns = tuple
    segment = list
    
elif conf.PYFORMS_MODE in ['WEB']:

    from pyforms_web.basewidget import BaseWidget
    from pyforms_web.organizers import no_columns, segment
    
    from pyforms_web.modeladmin import ModelAdmin
    from pyforms_web.modeladmin import ViewFormAdmin
    from pyforms_web.modeladmin import EditFormAdmin



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"


from numpy import *
import cv2, math

def biggestContour(contours, howmany=1):
    biggest = []
    for blob in contours: area = cv2.contourArea(blob); biggest.append( (area, blob) )
    if len(biggest)==0: return None
    biggest = sorted( biggest, key=lambda x: -x[0])
    if howmany==1: return biggest[0][1]
    return [x[1] for x in biggest[:howmany] ]

def getBiggestContour(image, howmany=1):
    (blobs, dummy) = cv2.findContours( image.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE )
    return biggestContour(blobs, howmany)

def cutBiggestContour(image, howmany=1):
    contour = getBiggestContour(image)

    mask = zeros_like(image);cv2.drawContours(mask, array([contour]), -1, (255,255,255), -1 )
    tmp = cv2.bitwise_and( image, mask )

    rect = cv2.boundingRect(contour)
    x, y, w, h = rect
    x, y, xx, yy = x,y,x+w,y+h
    return tmp[y:yy, x:xx]


def rotate_image(image, angle):
    """
    Rotates the given image about it's centre
    """

    image_size = (image.shape[1], image.shape[0])
    image_center = tuple(array(image_size) / 2)

    rot_mat = vstack([cv2.getRotationMatrix2D(image_center, angle, 1.0), [0, 0, 1]])
    trans_mat = identity(3)

    w2 = image_size[0] * 0.5
    h2 = image_size[1] * 0.5

    rot_mat_notranslate = matrix(rot_mat[0:2, 0:2])

    tl = (array([-w2, h2]) * rot_mat_notranslate).A[0]
    tr = (array([w2, h2]) * rot_mat_notranslate).A[0]
    bl = (array([-w2, -h2]) * rot_mat_notranslate).A[0]
    br = (array([w2, -h2]) * rot_mat_notranslate).A[0]

    x_coords = [pt[0] for pt in [tl, tr, bl, br]]
    x_pos = [x for x in x_coords if x > 0]
    x_neg = [x for x in x_coords if x < 0]

    y_coords = [pt[1] for pt in [tl, tr, bl, br]]
    y_pos = [y for y in y_coords if y > 0]
    y_neg = [y for y in y_coords if y < 0]

    right_bound = max(x_pos)
    left_bound = min(x_neg)
    top_bound = max(y_pos)
    bot_bound = min(y_neg)

    new_w = int(abs(right_bound - left_bound))
    new_h = int(abs(top_bound - bot_bound))
    new_image_size = (new_w, new_h)

    new_midx = new_w * 0.5
    new_midy = new_h * 0.5

    dx = int(new_midx - w2)
    dy = int(new_midy - h2)

    trans_mat = getTranslationMatrix2d(dx, dy)
    affine_mat = (matrix(trans_mat) * matrix(rot_mat))[0:2, :]
    result = cv2.warpAffine(image, affine_mat, new_image_size, flags=cv2.INTER_LINEAR)

    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

from pyforms_gui.controls.control_dockwidget import ControlDockWidget
from AnyQt.QtWidgets import QMainWindow, QDockWidget, QAction, QApplication, QToolBar
from AnyQt           import QtCore, _api
from AnyQt.QtGui     import QIcon
from confapp         import conf
import sys, os, platform, logging


logger = logging.getLogger(__name__)

class StandAloneContainer(QMainWindow):
    def __init__(self, ClassObject):
        super(QMainWindow, self).__init__()

        w = ClassObject()
        w.app_main_window = self
        self._widget = w

        if len(w.mainmenu) > 0:
            w._mainmenu = self.__initMainMenu(w.mainmenu)

        w.init_form()

        if _api.USED_API == _api.QT_API_PYQT5:
            self.layout().setContentsMargins(conf.PYFORMS_MAINWINDOW_MARGIN,conf.PYFORMS_MAINWINDOW_MARGIN,conf.PYFORMS_MAINWINDOW_MARGIN,conf.PYFORMS_MAINWINDOW_MARGIN)
        elif _api.USED_API == _api.QT_API_PYQT4:
            self.layout().setMargin(conf.PYFORMS_MAINWINDOW_MARGIN)

        self.setCentralWidget(w)
        self.setWindowTitle(w.title)

        docks = {}
        for name, item in w.controls.items():
            if isinstance(item, ControlDockWidget):
                if item.side not in docks:
                    docks[item.side] = []
                docks[item.side].append((name, item))
            
        #Construct the toolbar
        if len(w.toolbar)>0:
            tb = QToolBar(self)
            self.addToolBar(tb)
            for item in w.toolbar:
                if isinstance(item, str) and item in ['|','-']:
                    tb.addSeparator()
                else:
                    tb.addWidget(item.form)
  

        for key, widgets in docks.items():
            side = QtCore.Qt.RightDockWidgetArea
            if key == 'left':
                side = QtCore.Qt.LeftDockWidgetArea
            elif key == 'right':
                side = QtCore.Qt.RightDockWidgetArea
            elif key == 'top':
                side = QtCore.Qt.TopDockWidgetArea
            elif key == 'bottom':
                side = QtCore.Qt.BottomDockWidgetArea
            else:
                side = QtCore.Qt.LeftDockWidgetArea

            if isinstance(widgets, list):
                widgets = sorted(widgets, key=lambda x: x[1].order)

                for name, widget in widgets:
                    dock = QDockWidget(self)
                    dock.setFeatures(
                        QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetClosable | QDockWidget.DockWidgetMovable)
                    dock.setObjectName(name)
                        
                    if _api.USED_API == _api.QT_API_PYQT5:
                        dock.setContentsMargins(0,0,0,0)
                        widget.form.layout().setContentsMargins(widget.margin,widget.margin,widget.margin,widget.margin)
                    elif _api.USED_API == _api.QT_API_PYQT4:
                        dock.setMargin(0)
                        widget.form.layout().setMargin(widget.margin)

                    # print dock.objectName(),1
                    #widget.parent = self
                    dock.setWidget(widget.form)
                    dock.setWindowTitle(widget.label)
                    widget.dock = dock
                    if not widget._show: dock.hide()

                    self.addDockWidget(side, dock)
            else:
                dock = QDockWidget(self)
                dock.setFeatures(
                    QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetClosable | QDockWidget.DockWidgetMovable)
                # dock.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea | QtCore.Qt.RightDockWidgetArea)
                
                if _api.USED_API == _api.QT_API_PYQT5:
                    widget.form.layout().setContentsMargins(widget.margin,widget.margin,widget.margin,widget.margin)
                elif _api.USED_API == _api.QT_API_PYQT5:
                    widget.form.layout().setMargin(widget.margin)

                #widget.parent = self

                # print dock.objectName(), 2
                dock.setObjectName(name)
                dock.setWidget(widget.form)
                self.addDockWidget(side, dock)
                dock.setWindowTitle(widget.label)
                widget.dock = dock
                if not widget._show: dock.hide()

        if conf.PYFORMS_STYLESHEET:

            stylesheet_files = [conf.PYFORMS_STYLESHEET]

            p = platform.system()
            if p == 'Windows' and conf.PYFORMS_STYLESHEET_WINDOWS:
                stylesheet_files.append(conf.PYFORMS_STYLESHEET_WINDOWS)
            elif p == 'Darwin' and conf.PYFORMS_STYLESHEET_DARWIN:
                stylesheet_files.append(conf.PYFORMS_STYLESHEET_DARWIN)
            elif p == 'Linux' and conf.PYFORMS_STYLESHEET_LINUX:
                stylesheet_files.append(conf.PYFORMS_STYLESHEET_LINUX)

            logger.debug('Import stylesheets: {0}'.format(stylesheet_files))
            self.loadStyleSheetFile(stylesheet_files)

    def keyReleaseEvent(self, evt):
        super().keyReleaseEvent(evt)
        self._widget.keyReleaseEvent(evt)

    def keyPressEvent(self, evt):
        super().keyPressEvent(evt)
        self._widget.keyPressEvent(evt)

    def mousePressEvent(self, evt):
        super().mousePressEvent(evt)
        self._widget.mousePressEvent(evt)

    def mouseReleaseEvent(self, evt):
        super().mouseReleaseEvent(evt)
        self._widget.mouseReleaseEvent(evt)

    def mouseMoveEvent(self, evt):
        super().mouseMoveEvent(evt)
        self._widget.mouseMoveEvent(evt)

    def mouseDoubleClickEvent(self, evt):
        super().mouseDoubleClickEvent(evt)
        self._widget.mouseDoubleClickEvent(evt)

    def closeEvent(self, event):
        super().closeEvent(event)
        self._widget.closeEvent(event)

    def __initMainMenu(self, options, keys={}):
        menubar = self.menuBar()
        for menuIndex, m in enumerate(options):
            for key, menus in m.items():
                menu = menubar.addMenu(key)
                for subMenuIndex, m1 in enumerate(menus):
                    if isinstance(m1, str) and m1 == "-":
                        menu.addSeparator()
                    else:
                        for text, func in m1.items():
                            if text != 'icon':
                                action = QAction(text, self)
                                if 'icon' in m1.keys():
                                    action.setIconVisibleInMenu(True)
                                    action.setIcon(QIcon(m1['icon']))
                                if func:
                                    action.triggered.connect(func)
                                    menu.addAction(action)
                                options[menuIndex][key][subMenuIndex][text] = action
                                break
        return options

    def loadStyleSheetFile(self, files):
        content = ''
        for filename in files:
            infile = open(filename, 'r')
            content += infile.read() + '\n'
            infile.close()

        self.setStyleSheet(content)


def execute_test_file(myapp):
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("test_file", help="File with the tests script")
    args = parser.parse_args()

    with open(args.test_file) as f:
        global_vars = globals()
        local_vars = locals()
        code = compile(f.read(), args.test_file, 'exec')
        exec(code, global_vars, local_vars)


def start_app(ClassObject, geometry=None, stylesheet=None, user_settings=None, parent_win=None):

    if parent_win is None:
        app = QApplication(sys.argv)

    from confapp import conf
    conf += 'pyforms_gui.resources_settings'
    if user_settings:
        conf += user_settings

    mainwindow = StandAloneContainer(ClassObject)

    if hasattr( conf, 'PYFORMS_MAIN_WINDOW_ICON_PATH'):
        mainwindow.setWindowIcon(QIcon(conf.PYFORMS_MAIN_WINDOW_ICON_PATH))

    myapp = mainwindow.centralWidget()

    if geometry is not None:
        mainwindow.show()
        mainwindow.setGeometry(*geometry)
    else:
        mainwindow.showMaximized()

    if conf.PYFORMS_QUALITY_TESTS_PATH is not None:
        import argparse
        parser = argparse.ArgumentParser()
        parser.add_argument("--test", help="File with the tests script")
        args = parser.parse_args()

        if args.test:
            TEST_PATH = os.path.join(conf.PYFORMS_QUALITY_TESTS_PATH, args.test)
            TEST_FILE_PATH = os.path.join(TEST_PATH, args.test + '.py')
            DATA_PATH = os.path.join(TEST_PATH, 'data', sys.platform)
            INPUT_DATA_PATH = os.path.join(DATA_PATH, 'input-data')
            OUTPUT_DATA_PATH = os.path.join(DATA_PATH, 'output-data')
            EXPECTED_DATA_PATH = os.path.join(DATA_PATH, 'expected-data')

            with open(TEST_FILE_PATH) as f:
                global_vars = {}  # globals()
                local_vars = locals()
                code = compile(f.read(), TEST_FILE_PATH, 'exec')

                exec(code, global_vars, local_vars)

    if parent_win is None:
        if stylesheet: app.setStyleSheet(stylesheet)
        app.exec_()

    return myapp



#!/usr/bin/python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import re

version = ''
with open('idtrackerai_app/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE).group(1)

setup(
    name='idtrackerai-app',
    version=version,
    description="""""",
    author=['Ricardo Ribeiro', 'Francisco Romero Ferrero'],
    author_email='idtrackerai@gmail.com, ricardojvr@gmail.com, paco.romero.ferrero@gmail.com',
    url='https://idtrackerai-app.readthedocs.org',
    packages=find_packages(),
    install_requires=[
        'coloredlogs',
        'idtrackerai',
        'pyforms-terminal'
    ],
    extras_require={
        'gui':      ['pyforms-gui',
                     'python-video-annotator',
                     'python-video-annotator-module-idtrackerai==0.6.64'],
        'only-gui': ['pyforms-gui'],
    },
    entry_points={
        'console_scripts': [
            'idtrackerai=idtrackerai_app.__main__:start',
        ],
    }
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlPlayer.ControlPlayer

"""
__author__ = "Ricardo Ribeiro"
__credits__ = ["Ricardo Ribeiro"]
__license__ = "MIT"
__version__ = "0.0"
__maintainer__ = "Ricardo Ribeiro"
__email__ = "ricardojvr@gmail.com"
__status__ = "Development"

import logging, platform, os, math
from .multiple_videocapture import MultipleVideoCapture

try:
    import cv2
except:
    raise Exception('OpenCV is not available. ControlPlayer will not be working')


from confapp 	 import conf
from AnyQt 			 import uic, _api
from AnyQt 			 import QtCore
from AnyQt.QtWidgets import QFrame
from AnyQt.QtWidgets import QApplication
from AnyQt.QtWidgets import QMainWindow
from AnyQt.QtWidgets import QMessageBox

from pyforms_gui.controls.control_base import ControlBase

if _api.USED_API == _api.QT_API_PYQT5:
    import platform
    if platform.system() == 'Darwin':
        from pyforms_gui.controls.control_player.VideoQt5GLWidget import VideoQt5GLWidget as VideoGLWidget
    else:
        from pyforms_gui.controls.control_player.VideoGLWidget 	 import VideoGLWidget

elif _api.USED_API == _api.QT_API_PYQT4:
    from pyforms_gui.controls.control_player.VideoGLWidget 		 import VideoGLWidget


logger = logging.getLogger(__name__)

class ControlPlayer(ControlBase, QFrame):

    def __init__(self, *args, **kwargs):
        self._video_widget = None  # GL widget

        QFrame.__init__(self)
        ControlBase.__init__(self, *args, **kwargs)

        self._multiple_files = kwargs.get('multiple_files', False)

        self._current_frame = None  # current frame image
        self._current_frame_index = None # current frame index

        self.process_frame_event = kwargs.get('process_frame_event', self.process_frame_event)

        self._speed = 1
        self.logger = logging.getLogger('pyforms')

        self._update_video_frame  = True # if true update the spinbox with the current frame
        self._update_video_slider = True  # if true update the slider with the current frame



    def init_form(self):
        # Get the current path of the file
        rootPath = os.path.dirname(__file__)

        # Load the UI for the self instance
        uic.loadUi(os.path.join(rootPath, "video.ui"), self)


        # Define the icon for the Play button
        self.videoPlay.setIcon(conf.PYFORMS_ICON_VIDEOPLAYER_PAUSE_PLAY)
        self.detach_btn.setIcon(conf.PYFORMS_ICON_VIDEOPLAYER_DETACH)

        self.detach_btn.clicked.connect(self.__detach_player_evt)

        self._video_widget = VideoGLWidget()
        self._video_widget._control = self
        self.videoLayout.addWidget(self._video_widget)
        self.videoPlay.clicked.connect(self.videoPlay_clicked)
        self.videoFrames.valueChanged.connect(self.video_frames_value_changed)
        self.videoProgress.valueChanged.connect(self.videoProgress_valueChanged)
        self.videoProgress.sliderReleased.connect(self.videoProgress_sliderReleased)
        self._timer = QtCore.QTimer(self)
        self._timer.timeout.connect(self.call_next_frame)

        self.form.horizontalSlider.valueChanged.connect(self.__rotateZ)
        self.form.verticalSlider.valueChanged.connect(self.__rotateX)

        self._current_frame = None

        self.view_in_3D = False

    ##########################################################################
    ############ FUNCTIONS ###################################################
    ##########################################################################

    def save_form(self, data, path=None):
        return data

    def load_form(self, data, path=None):
        pass

    def hide(self):
        QFrame.hide(self)

    def show(self):
        QFrame.show(self)

    def play(self):
        """
        Play the video.
        :return:
        """
        try:
            self.videoPlay.setChecked(True)
            self._timer.start( 1000.0/float(self.fps+1) )
        except Exception as e:
            self.videoPlay.setChecked(False)
            logger.error(e, exc_info=True)

    def stop(self):
        """
        Stop the video
        :return:
        """
        self.videoPlay.setChecked(False)
        self._timer.stop()

    def toggle_playing(self):
        """
        Play or pause the video.
        :return:
        """
        if self.is_playing:
            self.stop()
        else:
            self.play()

    def refresh(self):
        """
        Refresh the frame in the player.
        :return:
        """
        if self._current_frame is not None:
            frame = self.process_frame_event(self._current_frame.copy())
            if isinstance(frame, list) or isinstance(frame, tuple):
                self._video_widget.paint(frame)
            else:
                self._video_widget.paint([frame])
        else:
            self._video_widget.paint(None)

    def jump_forward(self):
        """
        Jump 20 seconds forward.
        :return:
        """
        self.video_index += 20 * self.fps
        self.call_next_frame()

    def jump_backward(self):
        """
        Jump 20 seconds backward.
        :return:
        """
        self.video_index -= 20 * self.fps
        self.call_next_frame()

    def back_one_frame(self):
        """
        Back one frame.
        :return:
        """
        self.video_index -= 2
        self.call_next_frame()

    def forward_one_frame(self):
        """
        Forward one frame.
        :return:
        """
        self.call_next_frame()


    def set_speed_1x(self):
        """
        Set video playing speed 1x.
        :return:
        """
        self.next_frame_step = 1
        self.video_widget.show_tmp_msg('Speed: 1x')

    def set_speed_2x(self):
        """
        Set video playing speed 2x.
        :return:
        """
        self.next_frame_step = 2
        self.video_widget.show_tmp_msg('Speed: 2x')

    def set_speed_3x(self):
        """
        Set video playing speed 3x.
        :return:
        """
        self.next_frame_step = 3
        self.video_widget.show_tmp_msg('Speed: 3x')

    def set_speed_4x(self):
        """
        Set video playing speed 4x.
        :return:
        """
        self.next_frame_step = 4
        self.video_widget.show_tmp_msg('Speed: 4x')

    def set_speed_5x(self):
        """
        Set video playing speed 5x.
        :return:
        """
        self.next_frame_step = 5
        self.video_widget.show_tmp_msg('Speed: 5x')

    def set_speed_6x(self):
        """
        Set video playing speed 6x.
        :return:
        """
        self.next_frame_step = 6
        self.video_widget.show_tmp_msg('Speed: 6x')

    def set_speed_7x(self):
        """
        Set video playing speed 7x.
        :return:
        """
        self.next_frame_step = 7
        self.video_widget.show_tmp_msg('Speed: 7x')

    def set_speed_8x(self):
        """
        Set video playing speed 8x.
        :return:
        """
        self.next_frame_step = 8
        self.video_widget.show_tmp_msg('Speed: 8x')

    def set_speed_9x(self):
        """
        Set video playing speed 9x.
        :return:
        """
        self.next_frame_step = 9
        self.video_widget.show_tmp_msg('Speed: 9x')

    ##########################################################################
    ############ EVENTS ######################################################
    ##########################################################################

    def process_frame_event(self, frame): return frame

    @property
    def double_click_event(self): return self._video_widget.onDoubleClick
    @double_click_event.setter
    def double_click_event(self, value): self._video_widget.onDoubleClick = value

    @property
    def click_event(self): return self._video_widget.onClick
    @click_event.setter
    def click_event(self, value):  self._video_widget.onClick = value

    @property
    def drag_event(self): return self._video_widget.onDrag
    @drag_event.setter
    def drag_event(self, value): self._video_widget.onDrag = value

    @property
    def end_drag_event(self): return self._video_widget.onEndDrag
    @end_drag_event.setter
    def end_drag_event(self, value): self._video_widget.onEndDrag = value

    @property
    def key_press_event(self):
        return self._video_widget.on_key_press
    @key_press_event.setter
    def key_press_event(self, value):
        self._video_widget.on_key_press = value

    @property
    def key_release_event(self): return self._video_widget.on_key_release
    @key_release_event.setter
    def key_release_event(self, value): self._video_widget.on_key_release = value

    ##########################################################################
    ############ PROPERTIES ##################################################
    ##########################################################################

    @property
    def video_widget(self): return self._video_widget

    @property
    def next_frame_step(self): return self._speed
    @next_frame_step.setter
    def next_frame_step(self, value): self._speed = value

    @property
    def view_in_3D(self): return self._video_widget.onEndDrag
    @view_in_3D.setter
    def view_in_3D(self, value):
        self.form.horizontalSlider.setVisible(value)
        self.form.verticalSlider.setVisible(value)

    @property
    def video_index(self): return int(self._value.get(1)) if self._value else None

    @video_index.setter
    def video_index(self, value):
        if value<0: value = 0
        if value>=self.max: value = self.max-1
        self._value.set(1, value)

    @property
    def max(self):
        if self._value is None or self._value=='':
            return 0
        return int(self._value.get(7))

    @property
    def frame(self): return self._current_frame

    @frame.setter
    def frame(self, value):
        if isinstance(value, list) or isinstance(value, tuple):
            self._video_widget.paint(value)
        elif value is not None:
            self._video_widget.paint([value])
        else:
            self._video_widget.paint(None)
        QApplication.processEvents()

    @property
    def fps(self):
        """
            Return the video frames per second
        """
        return self._value.get(5)

    @property
    def help_text(self): return self._video_widget._helpText

    @help_text.setter
    def help_text(self, value): self._video_widget._helpText = value

    @property
    def form(self): return self

    @property
    def frame_width(self): return self._value.get(3)

    @property
    def frame_height(self): return self._value.get(4)

    @property
    def is_playing(self): return self._timer.isActive()

    @property
    def value(self): return ControlBase.value.fget(self)

    @property
    def multiple_files(self):
        return isinstance(self._value, MultipleVideoCapture)

    @value.setter
    def value(self, value):
        self.form.setUpdatesEnabled(False)
        if value is None:
            self.stop()
            self.videoControl.setEnabled(False)
            self.refresh()
        self._video_widget.reset()

        if value == 0:
            self._value = cv2.VideoCapture(0)
        elif isinstance(value, str) and value:

            open_multiplefiles = self._multiple_files

            if open_multiplefiles:
                open_multiplefiles = len(MultipleVideoCapture.search_files(value))>0

            if open_multiplefiles:
                msg = "Multiple files were found with the same folder, do you wish to combine then in a single video with the following order?\n\n"
                for filepath in MultipleVideoCapture.search_files(value):
                    msg += "- {filename}\n".format(filename=os.path.basename(filepath))

                reply = QMessageBox(
                    QMessageBox.Question,
                    'Open multiple files',
                    msg,
                    QMessageBox.No | QMessageBox.Yes
                ).exec_()

                if reply == QMessageBox.Yes:
                    open_multiplefiles = True
                else:
                    open_multiplefiles = False

            if open_multiplefiles:
                self._value = MultipleVideoCapture(value)
            else:
                self._value = cv2.VideoCapture(value)
        else:
            self._value = value

        if self._value and value != 0:
            self.videoProgress.setMinimum(0)
            self.videoProgress.setValue(0)
            self.videoProgress.setMaximum(
                self._value.get(7))
            self.videoFrames.setMinimum(0)
            self.videoFrames.setValue(0)
            self.videoFrames.setMaximum(
                self._value.get(7))

        if self._value:
            self.videoControl.setEnabled(True)

        self.refresh()
        self.form.setUpdatesEnabled(True)


    ##########################################################################
    ############ PRIVATE FUNCTIONS ###########################################
    ##########################################################################

    def __rotateX(self):
        self._video_widget.rotateX = self.form.verticalSlider.value()
        self.refresh()

    def __rotateZ(self):
        self._video_widget.rotateZ = self.form.horizontalSlider.value()
        self.refresh()





    def call_next_frame(self, update_slider=True, update_number=True, increment_frame=True):
        # move the player to the next frame
        self.form.setUpdatesEnabled(False)

        self._current_frame_index = self.video_index

        # if the player is not visible, stop
        if not self.visible:
            self.stop()
            self.form.setUpdatesEnabled(True)
            return

        # if no video is selected
        if self.value is None:
            self._current_frame = None
            self._current_frame_index = None
            return

        # read next frame
        (success, self._current_frame) = self.value.read()

        # increment frame index if the step is bigger than 1
        if increment_frame and self.next_frame_step > 1:
            self.video_index += self.next_frame_step

        # no frame available. leave the function
        if not success:
            self.stop()
            self.form.setUpdatesEnabled(True)
            return

        frame = self.process_frame_event(
            self._current_frame.copy()
        )

        # draw the frame
        if isinstance(frame, list) or isinstance(frame, tuple):
            self._video_widget.paint(frame)
        else:
            self._video_widget.paint([frame])

        if not self.videoProgress.isSliderDown():

            if update_slider and self._update_video_slider:
                self._update_video_slider = False
                self.videoProgress.setValue(self._current_frame_index)
                self._update_video_slider = True

            if update_number:
                self._update_video_frame = False
                self.videoFrames.setValue(self._current_frame_index)
                self._update_video_frame = True

        self.form.setUpdatesEnabled(True)


    def __detach_player_evt(self):
        """
        Called by the detach button
        """
        self._old_layout = self.parentWidget().layout()
        self._old_layout_index = self._old_layout.indexOf(self)
        self._detach_win = QMainWindow(parent=self.parent)
        self._detach_win.setWindowTitle('Player')
        self._detach_win.setCentralWidget(self)
        self.detach_btn.hide()
        self._detach_win.closeEvent = self.__detach_win_closed_evt
        self._detach_win.show()

    def __detach_win_closed_evt(self, event):
        """
        Called when the detached window is closed
        """
        self._old_layout.insertWidget(self._old_layout_index, self)
        self.detach_btn.show()
        self._detach_win.close()
        del self._detach_win

    def videoPlay_clicked(self):
        """Slot for Play/Pause functionality."""
        if self.is_playing:
            self.stop()
        else:
            self.play()

    def convertFrameToTime(self, totalMilliseconds):
        # totalMilliseconds = totalMilliseconds*(1000.0/self._value.get(5))
        if math.isnan(totalMilliseconds): return 0, 0, 0
        totalseconds = int(totalMilliseconds / 1000)
        minutes = int(totalseconds / 60)
        seconds = totalseconds - (minutes * 60)
        milliseconds = totalMilliseconds - (totalseconds * 1000)
        return (minutes, seconds, milliseconds)

    def videoProgress_valueChanged(self):
        milli = self._value.get(0)
        (minutes, seconds, milliseconds) = self.convertFrameToTime(milli)
        self.videoTime.setText(
            "%02d:%02d:%03d" % (minutes, seconds, milliseconds))



    def videoProgress_sliderReleased(self):

        if not self.is_playing and self._update_video_slider:
            new_index = self.videoProgress.value()
            self._value.set(1, new_index)
            self.call_next_frame(update_slider=False, increment_frame=False)

    def video_frames_value_changed(self, pos):

        if not self.is_playing and self._update_video_frame:
            self._value.set(1, pos) # set the video position
            self.call_next_frame(update_number=False, increment_frame=False)




#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import AnyQt

from AnyQt.QtWidgets import QFrame
from AnyQt.QtWidgets import QVBoxLayout
from AnyQt.QtWidgets import QTabWidget
from AnyQt.QtWidgets import QSplitter
from AnyQt.QtWidgets import QHBoxLayout
from AnyQt.QtWidgets import QSpacerItem
from AnyQt.QtWidgets import QSizePolicy
from AnyQt.QtWidgets import QLabel
from AnyQt.QtGui import QFont
from AnyQt.QtWidgets import QFileDialog
from AnyQt import QtCore, _api

from pyforms_gui.controls.control_base import ControlBase

from AnyQt.QtWidgets import QMessageBox, QInputDialog

from .organizers import *


class BaseWidget(QFrame):
    """
    The class implements the most basic widget or window.
    """

    def __init__(self, *args, **kwargs):
        title = kwargs.get('title', args[0] if len(args) > 0 else '')

        parent_win = kwargs.get('parent_win', kwargs.get('parent_widget', None))
        win_flag = kwargs.get('win_flag', None)

        self._parent_widget = parent_win

        if parent_win is not None and win_flag is None:
            win_flag = QtCore.Qt.Dialog

        QFrame.__init__(self) if parent_win is None else QFrame.__init__(self, parent_win, win_flag)

        layout = QVBoxLayout()
        self.setLayout(layout)
        
        if _api.USED_API == _api.QT_API_PYQT5:
            layout.setContentsMargins(0, 0, 0, 0)
        elif _api.USED_API == _api.QT_API_PYQT4:
            layout.setMargin(0)

        self.title = title

        self.title = title
        self.has_progress = False

        self.toolbar = []
        self._mainmenu = []
        self._splitters = []
        self.vlayouts = []
        self.hlayouts = []
        self._tabs = []
        self._formset = None
        self._formLoaded = False
        self.uid = id(self)

        self.setAccessibleName('BaseWidget')

    ##########################################################################
    ############ FUNCTIONS  ##################################################
    ##########################################################################

    def init_form(self):
        """
        Generate the module Form
        """
        if not self._formLoaded:

            allparams = self.controls
            for key, param in allparams.items():
                param.parent = self
                param.name = key

            if self._formset is not None:
                control = self.generate_panel(self._formset)
                self.layout().addWidget(control)
            else:
                for key, param in allparams.items():
                    self.layout().addWidget(param.form)
            self._formLoaded = True

    def set_margin(self, margin):
        if _api.USED_API == _api.QT_API_PYQT5:
            self.layout().setContentsMargins(margin,margin,margin,margin)
        elif _api.USED_API == _api.QT_API_PYQT4:
            self.layout().setMargin(margin)

    def generate_tabs(self, formsetdict):
        """
        Generate QTabWidget for the module form
        @param formset: Tab form configuration
        @type formset: dict
        """
        tabs = QTabWidget(self)
        for key, item in sorted(formsetdict.items()):
            ctrl = self.generate_panel(item)
            tabs.addTab(ctrl, key[key.find(':') + 1:])
        return tabs

    def generate_panel(self, formset):
        """
        Generate a panel for the module form with all the controls
        formset format example: [('_video', '_arenas', '_run'), {"Player":['_threshold', "_player", "=", "_results", "_query"], "Background image":[(' ', '_selectBackground', '_paintBackground'), '_image']}, "_progress"]
        tuple: will display the controls in the same horizontal line
        list: will display the controls in the same vertical line
        dict: will display the controls in a tab widget
        '||': will split the controls in a horizontal line
        '=': will split the controls in a vertical line
        @param formset: Form configuration
        @type formset: list
        """
        control = None
        if '=' in formset or isinstance( formset, hsplitter):
            control = QSplitter(QtCore.Qt.Vertical)
            index = list(formset).index('=')
            first_panel = self.generate_panel(formset[0:index])
            second_panel = self.generate_panel(formset[index+1:])
            control.addWidget(first_panel)
            control.addWidget(second_panel)
            self._splitters.append(control)
            return control
        elif '||' in formset or isinstance( formset, vsplitter):
            control = QSplitter(QtCore.Qt.Horizontal)
            index = list(formset).index('||')
            first_panel = self.generate_panel(formset[0:index])
            second_panel = self.generate_panel(formset[index+1:])
            control.addWidget(first_panel)
            control.addWidget(second_panel)

            if isinstance(formset, vsplitter):
                sizes = [formset.left_width, formset.right_width]
                control.setSizes(sizes)
            self._splitters.append(control)
            return control
        control = QFrame(self)
        layout = None
        if isinstance(formset, (tuple, no_columns)):
            layout = QHBoxLayout()
            self.hlayouts.append(layout)
            for row in formset:
                if isinstance(row, (list, tuple, vsplitter, hsplitter, no_columns, segment)):
                    panel = self.generate_panel(row)
                    layout.addWidget(panel)
                elif row == " ":
                    spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
                    layout.addItem(spacer)
                elif type(row) is dict:
                    c = self.generate_tabs(row)
                    layout.addWidget(c)
                    self._tabs.append(c)
                else:
                    self._handle_text(layout, row)
        elif isinstance(formset, (list, segment)):
            layout = QVBoxLayout()
            self.vlayouts.append(layout)
            for row in formset:
                if isinstance(row, (list, tuple, vsplitter, hsplitter, segment, no_columns) ):
                    panel = self.generate_panel(row)
                    layout.addWidget(panel)
                elif row == " ":
                    spacer = QSpacerItem(
                        20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
                    layout.addItem(spacer)
                elif type(row) is dict: 
                    c = self.generate_tabs(row)
                    layout.addWidget(c)
                    self._tabs.append(c)
                else:
                    self._handle_text(layout, row)

        if _api.USED_API == _api.QT_API_PYQT5:
            layout.setContentsMargins(0, 0, 0, 0)
        elif _api.USED_API == _api.QT_API_PYQT4:
            layout.setMargin(0)
            
        control.setLayout(layout)
        return control

    def _handle_text(self, layout, row):
        param = self.controls.get(row, None)
        if param is None:
            label = QLabel()
            label.setOpenExternalLinks(True)
            label.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)

            if row.startswith('info:'):
                label.setText(row[5:])
                font = QFont()
                font.setPointSize(10)
                label.setFont(font)
                label.setAccessibleName('info')
            elif row.startswith('h1:'):
                label.setText(row[3:])
                font = QFont()
                font.setPointSize(17)
                font.setBold(True)
                label.setFont(font)
                label.setAccessibleName('h1')
            elif row.startswith('h2:'):
                label.setText(row[3:])
                font = QFont()
                font.setPointSize(16)
                font.setBold(True)
                label.setFont(font)
                label.setAccessibleName('h2')
            elif row.startswith('h3:'):
                label.setText(row[3:])
                font = QFont()
                font.setPointSize(15)
                font.setBold(True)
                label.setFont(font)
                label.setAccessibleName('h3')
            elif row.startswith('h4:'):
                label.setText(row[3:])
                font = QFont()
                font.setPointSize(14)
                font.setBold(True)
                label.setFont(font)
                label.setAccessibleName('h4')
            elif row.startswith('h5:'):
                label.setText(row[3:])
                font = QFont()
                font.setPointSize(12)
                font.setBold(True)
                label.setFont(font)
                label.setAccessibleName('h5')
            else:
                label.setText(row)
                label.setAccessibleName('msg')
            label.setToolTip(label.text())
            layout.addWidget(label)
        else:
            param.parent = self
            param.name = row
            layout.addWidget(param.form)

    def show(self):
        self.init_form()
        super(BaseWidget, self).show()

    def save_form(self, data={}, path=None):
        allparams = self.controls

        if hasattr(self, 'load_order'):
            for name in self.load_order:
                param = allparams[name]
                data[name] = {}
                param.save_form(data[name])
        else:
            for name, param in allparams.items():
                data[name] = {}
                param.save_form(data[name])
        return data

    def load_form(self, data, path=None):
        allparams = self.controls

        if hasattr(self, 'load_order'):
            for name in self.load_order:
                param = allparams[name]
                if name in data:
                    param.load_form(data[name])
        else:
            for name, param in allparams.items():
                if name in data:
                    param.load_form(data[name])
                # self.init_form()

    def save_window(self):
        allparams = self.controls
        data = {}
        self.save_form(data)

        filename, _ = QFileDialog.getSaveFileName(self, 'Select file')
        if filename:
            with open(filename, 'w') as output_file:
                json.dump(data, output_file)

    def load_form_filename(self, filename):
        with open(filename, 'r') as pkl_file:
            project_data = json.load(pkl_file)
        data = dict(project_data)
        self.load_form(data)

    def load_window(self):
        filename, _ = QFileDialog.getOpenFileNames(self, 'Select file')
        self.load_form_filename(str(filename[0]))

    def close(self):
        super(BaseWidget, self).close()

    def input_text(self, msg, title='', default=None):
        text, ok = QInputDialog.getText(self, title, msg, text=default)
        if ok:
            return str(text)
        else:
            return None

    def input_double(self, msg, title='', default=0, min=-2147483647, max=2147483647, decimals=1):
        text, ok = QInputDialog.getDouble(self, title, msg, value=default, min=min, max=max, decimals=decimals)
        if ok:
            return float(text)
        else:
            return None
        
    def input_int(self, msg, title='', default=0, min=-2147483647, max=2147483647):
        text, ok = QInputDialog.getInt(self, title, msg, value=default, min=min, max=max)
        if ok:
            return int(text)
        else:
            return None

    def question(self, msg, title=None, buttons=['no', 'yes']):
        btns = None
        for btn in buttons:
            if btn.lower() == 'cancel':
                b = QMessageBox.Cancel
            elif btn.lower() == 'no':
                b = QMessageBox.No
            elif btn.lower() == 'yes':
                b = QMessageBox.Yes
            elif btn.lower() == 'no_all':
                b = QMessageBox.NoToAll
            elif btn.lower() == 'yes_all':
                b = QMessageBox.YesToAll

            if btns is None: 
                btns = b
            else:
                btns |= b

        m = QMessageBox(QMessageBox.Question, title, msg, btns)
        reply = m.exec_()

        if reply == QMessageBox.Cancel:
            return 'cancel'
        elif reply == QMessageBox.No:
            return 'no'
        elif reply == QMessageBox.Yes:
            return 'yes'
        elif reply == QMessageBox.NoToAll:
            return 'no_all'
        elif reply == QMessageBox.YesToAll:
            return 'yes_all'

        return None

    def message(self, msg, title=None, msg_type=None):
        if msg_type == 'success':
            m = QMessageBox(QMessageBox.NoIcon, title, msg) 
        elif msg_type == 'info':
            m = QMessageBox(QMessageBox.Information, title, msg)    
        elif msg_type == 'warning':
            m = QMessageBox(QMessageBox.Warning, title, msg) 
        elif msg_type == 'error':
            m = QMessageBox(QMessageBox.Critical, title, msg)
        elif msg_type == 'about':
            m = QMessageBox(QMessageBox.Question, title, msg) 
        elif msg_type == 'aboutQt':
            m = QMessageBox(QMessageBox.Question, title, msg)
        else:
            m = QMessageBox(QMessageBox.NoIcon, title, msg)

        m.exec_()

    def success(self,   msg, title=None):   self.message(msg, title, msg_type='success')
    def info(self,      msg, title=None):   self.message(msg, title, msg_type='info')
    def warning(self,   msg, title=None):   self.message(msg, title, msg_type='warning');
    def alert(self,     msg, title=None):   self.message(msg, title, msg_type='error')
    def critical(self,  msg, title=None):   self.message(msg, title, msg_type='error')
    def about(self,     msg, title=None):   self.message(msg, title, msg_type='about')
    def aboutQt(self,   msg, title=None):   self.message(msg, title, msg_type='aboutQt')

    def message_popup(self, msg, title='', buttons=None, handler=None, msg_type='success'):
        pass

    def success_popup(self, msg, title='', buttons=None, handler=None):
        return self.message_popup(msg, title, buttons, handler, msg_type='success')

    def info_popup(self, msg, title='', buttons=None, handler=None):
        return self.message_popup(msg, title, buttons, handler, msg_type='info')

    def warning_popup(self, msg, title='', buttons=None, handler=None):
        return self.message_popup(msg, title, buttons, handler, msg_type='warning')

    def alert_popup(self, msg, title='', buttons=None, handler=None):
        return self.message_popup(msg, title, buttons, handler, msg_type='alert')

    ##########################################################################
    ############ GUI functions ###############################################
    ##########################################################################

    def set_margin(self, margin):
        if AnyQt.USED_API == 'pyqt5':
            self.layout().setContentsMargins(margin, margin, margin, margin)
        else:
            self.layout().setMargin(margin)

    ##########################################################################
    ############ EVENTS ######################################################
    ##########################################################################

    def before_close_event(self):
        """ 
        Do something before closing widget 
        Note that the window will be closed anyway    
        """
        pass

    ##########################################################################
    ############ Properties ##################################################
    ##########################################################################

    @property
    def controls(self):
        """
        Return all the form controls from the the module
        """
        result = {}
        for name, var in vars(self).items():
            try:
                if isinstance(var, ControlBase):
                    result[name] = var
            except:
                pass
        return result

    ############################################################################
    ############ GUI Properties ################################################
    ############################################################################

    @property
    def form_has_loaded(self):
        return self._formLoaded

    @property 
    def form(self): 
        return self 
    
    @property
    def visible(self):
        return self.isVisible()

    @property
    def mainmenu(self):
        return self._mainmenu

    @mainmenu.setter
    def mainmenu(self, value):
        self._mainmenu = value

    @property
    def controls(self):
        """
        Return all the form controls from the the module
        """
        result = {}
        for name, var in vars(self).items():
            try:
                if isinstance(var, ControlBase):
                    result[name] = var
            except:
                pass
        return result

    ############################################################################
    ############ GUI Properties ################################################
    ############################################################################

    @property
    def form_has_loaded(self):
        return self._formLoaded

    @property
    def parent_widget(self):
        return self._parent_widget

    @property
    def form(self):
        return self

    @property
    def title(self):
        return self.windowTitle()

    @title.setter
    def title(self, value):
        self.setWindowTitle(value)

    @property
    def formset(self):
        return self._formset

    @formset.setter
    def formset(self, value):
        self._formset = value

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, value):
        self._uid = value

    @property
    def visible(self):
        return self.isVisible()

    ##########################################################################
    ############ PRIVATE FUNCTIONS ###########################################
    ##########################################################################
    
    def closeEvent(self, event):
        self.before_close_event()
        super(BaseWidget, self).closeEvent(event)



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = "Ricardo Ribeiro"
__credits__ = ["Ricardo Ribeiro"]
__license__ = "MIT"
__version__ = "0.0"
__maintainer__ = "Ricardo Ribeiro"
__email__ = "ricardojvr@gmail.com"
__status__ = "Development"

import visvis as vv, numpy as np
from pyforms_gui.controls.control_base import ControlBase
from visvis                           import Point, Pointset
from AnyQt                            import _api
from AnyQt.QtWidgets                  import QWidget, QVBoxLayout, QSizePolicy

class ControlVisVisVolume(ControlBase):
    def init_form(self):
        self._form = QWidget()
        layout = QVBoxLayout()
        
        if _api.USED_API == _api.QT_API_PYQT5:
            layout.setContentsMargins(0,0,0,0)
        elif _api.USED_API == _api.QT_API_PYQT4:
            layout.setMargin(0)

        self._form.setLayout(layout)
        self._app = vv.use('pyqt5')
        self._app.Create()
        self._first = True

        Figure = self._app.GetFigureClass()
        self._fig = Figure(self._form)
        vv.figure(self._fig.nr)

        policy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        widget = self._fig._widget
        widget.setSizePolicy(policy)
        widget.setMinimumSize(100, 100)

        layout.addWidget(widget)

        self._colorMap = vv.CM_AUTUMN
        self._colors_limits = None
        super(ControlVisVisVolume, self).init_form()

    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def color_map(self):
        return self._colorMap

    @color_map.setter
    def color_map(self, value):
        self._colorMap = value
        self.refresh()

    def refresh(self):
        if len(self._value) > 1:
            vv.figure(self._fig.nr)

            a = vv.gca()
            view = a.GetView()
            a.Clear()
            vv.volshow3(self._value, renderStyle='mip', cm=self._colorMap, clim=self._colors_limits)
            if not self._first:
                a = vv.gca()
                a.SetView(view)

            self._first = False

    @property
    def value(self):
        return None

    @value.setter
    def value(self, value):
        self._value = value
        self.refresh()

    @property
    def colors_limits(self):
        return self._colors_limits

    @colors_limits.setter
    def colors_limits(self, value):
        self._colors_limits = value
        self.refresh()

    @property
    def visvis(self):
        return vv



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__ = "4.904.149"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"

from confapp import conf

try:
    import local_settings
    conf += local_settings
except:
    pass

from . import settings as pyforms_settings
conf += pyforms_settings

import logging

logging.basicConfig(
    level=conf.PYFORMS_LOG_HANDLER_LEVEL,
    format=conf.PYFORMS_LOG_FORMAT,
    handlers=[logging.StreamHandler()]
)



from pyforms_gui.basewidget  import BaseWidget
from pyforms_gui.controls.control_progress import ControlProgress
from pyforms_gui.controls.control_button import ControlButton
from pyforms_gui.controls.control_list import ControlList
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_slider import ControlSlider
from pyforms_gui.controls.control_textarea import ControlTextArea
from pyforms_gui.controls.control_text import ControlText
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_label import ControlLabel
from AnyQt.QtWidgets  import QMessageBox
import traceback

class Graph2Event(BaseWidget):

	def __init__(self, timeline=None):
		super(Graph2Event, self).__init__('Graph to event', parent_win=timeline)
		self.setContentsMargins(10, 10, 10, 10)
		self._timeline = timeline

		# Definition of the forms fields
		self._graphs_list = ControlList('Graphs list (try double click)')
		self._equation    = ControlTextArea('Equation')
		self._eventname   = ControlText('Event name', 'New event')
		self._rownumber   = ControlNumber('Row number', 0,0,1000)
		self._mindiff 	  = ControlNumber('Minimum of frames', 0, 0, 1000000)
		self._genevts_btn = ControlButton('Generate events')
		
		self._formset = [
			(
				['_graphs_list'],
				'||', 
				[
					('_eventname','_rownumber','_mindiff'),
					'_equation',
					'_genevts_btn',
				]
			),
		]

		self._graphs_list.cell_double_clicked_event = self.__cell_double_clicked_evt 
		self._graphs_list.readonly = True
		self._graphs_list.select_entire_row = True
		self._genevts_btn.value = self.__generage_events_evt



	def __add__(self, other):
		self._graphs_list += [other.name]
		return self

	def __sub__(self, other):
		self._graphs_list -= other
		return self

	def rename_graph(self, graph_index, newname):
		self._graphs_list.set_value(graph_index, 0, newname)

	@property
	def graphs(self):
		return self._timeline._graphs


	def __cell_double_clicked_evt(self, row, column):
		if len(self._equation.value.strip())==0:
			self._equation.value += '[{0}]'.format( self._graphs_list.value[row][column] )
		else:
			self._equation.value += ' and [{0}]'.format( self._graphs_list.value[row][column] )


	def __generage_events_evt(self):
		if len(self._eventname.value.strip())==0: 
			QMessageBox.warning(
				self, "Attention!", 'The event name cannot be empty')
			return

		if len(self._equation.value.strip())==0: 
			QMessageBox.warning(
				self, "Attention!", 'The equation cannot be empty')
			return 

		max_frame = 0
		equation  = self._equation.value
		for i, values in enumerate(self._graphs_list.value):
			graphname = '[{0}]'.format(values[0])

			if graphname in equation: max_frame = max(max_frame, len(self.graphs[i]))

			equation  = equation.replace(graphname, 'graphs[{0}][i]'.format(i) )
		
		graphs = self.graphs

		last_index = None
		last_value = False

		try:		
			for i in range(max_frame):
				try:
					value = eval(equation)
				except:
					value = False

				if not last_value and bool(value):
					last_index = i
					last_value = True
				
				if last_value and not bool(value):
					if (i-1-last_index)>=self._mindiff.value:
						self._timeline.add_event(last_index, i-1, title=self._eventname.value, row=int(self._rownumber.value) )
					last_value = False
					last_index = None

			if last_value and (max_frame-last_index) >= self._mindiff.value:
				self._timeline.add_event(last_index, max_frame, title=self._eventname.value, row=int(self._rownumber.value) )
		except Exception as e:
			traceback.print_exc()
			QMessageBox.warning( self, "Error!", str(e) )
			

		self._timeline.repaint()

	
##################################################################################################################
##################################################################################################################
##################################################################################################################

# Execute the application
if __name__ == "__main__":
	import pyforms
	pyforms.start_app(Graph2Event, geometry=(0, 0, 600, 400))
